import datetime as dt
import logging
from collections import defaultdict, deque
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Any,
    Deque,
    Dict,
    List,
    Literal,
    Optional,
    Tuple,
    Union,
    cast,
)

import numpy as np
import pandas as pd

from .akquant import (
    Bar,
    ExecutionMode,
    Order,
    StrategyContext,
    Tick,
    TimeInForce,
)
from .sizer import FixedSize, Sizer
from .strategy_events import (
    on_bar_event as _on_bar_event_impl,
)
from .strategy_events import (
    on_tick_event as _on_tick_event_impl,
)
from .strategy_events import (
    on_timer_event as _on_timer_event_impl,
)
from .strategy_framework_hooks import (
    call_user_callback as _call_user_callback_impl,
)
from .strategy_framework_hooks import (
    dispatch_shutdown_hooks as _dispatch_shutdown_hooks_impl,
)
from .strategy_framework_hooks import (
    ensure_framework_state as _ensure_framework_state_impl,
)
from .strategy_history import get_history as _get_history_impl
from .strategy_history import get_history_df as _get_history_df_impl
from .strategy_history import get_rolling_data as _get_rolling_data_impl
from .strategy_history import set_history_depth as _set_history_depth_impl
from .strategy_history import set_rolling_window as _set_rolling_window_impl
from .strategy_logging import log as _log_impl
from .strategy_ml import auto_configure_model as _auto_configure_model_impl
from .strategy_ml import on_train_signal as _on_train_signal_impl
from .strategy_order_events import (
    check_order_events as _check_order_events_impl,
)
from .strategy_order_events import (
    key_value as _key_value_impl,
)
from .strategy_order_events import (
    remember_trade_key as _remember_trade_key_impl,
)
from .strategy_order_events import (
    trade_dedupe_cache_limit as _trade_dedupe_cache_limit_impl,
)
from .strategy_order_events import (
    trade_event_key as _trade_event_key_impl,
)
from .strategy_position import Position
from .strategy_scheduler import add_daily_timer as _add_daily_timer_impl
from .strategy_scheduler import schedule as _schedule_impl
from .strategy_time import format_time as _format_time_impl
from .strategy_time import now as _now_impl
from .strategy_time import to_local_time as _to_local_time_impl
from .strategy_trading_api import (
    buy as _buy_impl,
)
from .strategy_trading_api import (
    buy_all as _buy_all_impl,
)
from .strategy_trading_api import (
    calculate_max_buy_qty as _calculate_max_buy_qty_impl,
)
from .strategy_trading_api import (
    cancel_all_orders as _cancel_all_orders_impl,
)
from .strategy_trading_api import (
    cancel_order as _cancel_order_impl,
)
from .strategy_trading_api import (
    close_position as _close_position_impl,
)
from .strategy_trading_api import (
    cover as _cover_impl,
)
from .strategy_trading_api import (
    get_account as _get_account_impl,
)
from .strategy_trading_api import (
    get_available_position as _get_available_position_impl,
)
from .strategy_trading_api import (
    get_cash as _get_cash_impl,
)
from .strategy_trading_api import (
    get_execution_capabilities as _get_execution_capabilities_impl,
)
from .strategy_trading_api import (
    get_open_orders as _get_open_orders_impl,
)
from .strategy_trading_api import (
    get_order as _get_order_impl,
)
from .strategy_trading_api import (
    get_portfolio_value as _get_portfolio_value_impl,
)
from .strategy_trading_api import (
    get_position as _get_position_impl,
)
from .strategy_trading_api import (
    get_positions as _get_positions_impl,
)
from .strategy_trading_api import (
    hold_bar as _hold_bar_impl,
)
from .strategy_trading_api import (
    order_target as _order_target_impl,
)
from .strategy_trading_api import (
    order_target_percent as _order_target_percent_impl,
)
from .strategy_trading_api import (
    order_target_value as _order_target_value_impl,
)
from .strategy_trading_api import (
    order_target_weights as _order_target_weights_impl,
)
from .strategy_trading_api import (
    resolve_symbol as _resolve_symbol_impl,
)
from .strategy_trading_api import (
    sell as _sell_impl,
)
from .strategy_trading_api import (
    short as _short_impl,
)
from .strategy_trading_api import (
    stop_buy as _stop_buy_impl,
)
from .strategy_trading_api import (
    stop_sell as _stop_sell_impl,
)
from .strategy_trading_api import (
    submit_order as _submit_order_impl,
)

if TYPE_CHECKING:
    from .indicator import Indicator
    from .ml.model import QuantModel


@dataclass
class StrategyRuntimeConfig:
    """策略运行时行为配置."""

    enable_precise_day_boundary_hooks: bool = False
    portfolio_update_eps: float = 0.0
    error_mode: Literal["raise", "continue", "legacy"] = "raise"
    re_raise_on_error: bool = True
    indicator_mode: Literal["incremental", "precompute"] = "precompute"

    def __post_init__(self) -> None:
        """校验并标准化配置."""
        self.portfolio_update_eps = float(self.portfolio_update_eps)
        if self.portfolio_update_eps < 0.0:
            raise ValueError("portfolio_update_eps must be >= 0")
        mode = str(self.error_mode).strip().lower()
        if mode not in {"raise", "continue", "legacy"}:
            raise ValueError("error_mode must be one of: raise, continue, legacy")
        self.error_mode = cast(Literal["raise", "continue", "legacy"], mode)
        indicator_mode = str(self.indicator_mode).strip().lower()
        if indicator_mode not in {"incremental", "precompute"}:
            raise ValueError("indicator_mode must be one of: incremental, precompute")
        self.indicator_mode = cast(Literal["incremental", "precompute"], indicator_mode)
        self.enable_precise_day_boundary_hooks = bool(
            self.enable_precise_day_boundary_hooks
        )
        self.re_raise_on_error = bool(self.re_raise_on_error)


class Strategy:
    """
    策略基类 (Base Strategy Class).

    采用事件驱动设计
    """

    ctx: Optional[StrategyContext]
    execution_mode: Optional[ExecutionMode]
    sizer: Sizer
    current_bar: Optional[Bar]
    current_tick: Optional[Tick]
    _history_depth: int
    # Rust maintains HistoryBuffer for indicator calculation.
    # Python side accesses it via self.ctx.history() (efficient copy).
    # No duplicate storage in Python.
    _precomputed_indicators: List["Indicator"]
    _incremental_indicators: Dict[str, Dict[str, Any]]
    _subscriptions: List[str]
    _last_prices: Dict[str, float]
    _rolling_train_window: int
    _rolling_step: int
    _bar_count: int
    _model_configured: bool
    model: Optional["QuantModel"]
    _known_orders: Dict[str, Order]
    _seen_trade_keys: set[Tuple[Any, ...]]
    _seen_trade_key_order: Deque[Tuple[Any, ...]]
    timezone: str = "Asia/Shanghai"
    warmup_period: int = 0
    _runtime_config: StrategyRuntimeConfig
    _last_event_type: str = ""  # "bar" or "tick"
    _hold_bars: "defaultdict[str, int]"
    _last_position_signs: "defaultdict[str, float]"
    _framework_last_session: Any
    _framework_last_local_date: Optional[dt.date]
    _framework_before_trading_done_date: Optional[dt.date]
    _framework_after_trading_done_date: Optional[dt.date]
    _framework_last_portfolio_state: Any
    _framework_portfolio_dirty: bool
    _framework_rejected_order_ids: set[str]
    _framework_stop_flushed: bool
    _framework_boundary_timers_registered: bool
    _trading_day_bounds: Dict[str, Tuple[int, int]]
    _oco_groups: Dict[str, set[str]]
    _oco_order_to_group: Dict[str, str]
    _pending_brackets: Dict[str, Dict[str, Any]]
    _order_group_seq: int

    _trading_days: List[pd.Timestamp]

    # Fee rates
    commission_rate: float
    min_commission: float
    stamp_tax_rate: float
    transfer_fee_rate: float
    lot_size: Any  # Can be int or Dict[str, int]

    def __new__(cls, *args: Any, **kwargs: Any) -> "Strategy":
        """Create a new Strategy instance."""
        instance = super().__new__(cls)
        # 初始化默认属性，确保 pickle 恢复前也有这些字段
        instance._is_restored = False
        instance.ctx = None
        instance.execution_mode = None
        instance.sizer = FixedSize(100)
        instance.current_bar = None
        instance.current_tick = None
        instance._precomputed_indicators = []
        instance._incremental_indicators = {}
        instance._subscriptions = []
        instance._last_prices = {}
        instance._known_orders = {}
        instance._seen_trade_keys = set()
        instance._seen_trade_key_order = deque()
        instance._hold_bars = defaultdict(int)
        instance._last_position_signs = defaultdict(float)
        instance.timezone = getattr(instance, "timezone", "Asia/Shanghai")
        raw_runtime_config = getattr(instance, "_runtime_config", None)
        class_enable_hooks = cls.__dict__.get(
            "enable_precise_day_boundary_hooks", False
        )
        class_portfolio_eps = cls.__dict__.get("portfolio_update_eps", 0.0)
        class_error_mode = cls.__dict__.get("error_mode", "raise")
        class_re_raise = cls.__dict__.get("re_raise_on_error", True)
        class_indicator_mode = cls.__dict__.get("indicator_mode", "precompute")
        if isinstance(raw_runtime_config, dict):
            instance.runtime_config = StrategyRuntimeConfig(**raw_runtime_config)
        elif isinstance(raw_runtime_config, StrategyRuntimeConfig):
            instance.runtime_config = StrategyRuntimeConfig(
                enable_precise_day_boundary_hooks=raw_runtime_config.enable_precise_day_boundary_hooks,
                portfolio_update_eps=raw_runtime_config.portfolio_update_eps,
                error_mode=raw_runtime_config.error_mode,
                re_raise_on_error=raw_runtime_config.re_raise_on_error,
                indicator_mode=raw_runtime_config.indicator_mode,
            )
        else:
            instance.runtime_config = StrategyRuntimeConfig(
                enable_precise_day_boundary_hooks=bool(class_enable_hooks),
                portfolio_update_eps=float(class_portfolio_eps),
                error_mode=cast(
                    Literal["raise", "continue", "legacy"], str(class_error_mode)
                ),
                re_raise_on_error=bool(class_re_raise),
                indicator_mode=cast(
                    Literal["incremental", "precompute"], str(class_indicator_mode)
                ),
            )
        instance._last_event_type = ""
        instance._trading_days = []

        # 历史数据配置
        instance._history_depth = 0
        instance.warmup_period = getattr(instance, "warmup_period", 0)

        # 滚动训练配置
        instance._rolling_train_window = 0
        instance._rolling_step = 0
        instance._bar_count = 0
        instance._model_configured = False
        instance._start_initialized = False

        # 初始化通常在 __init__ 中的属性，允许子类省略 super().__init__()
        instance.model = None

        # 默认费率配置
        instance.commission_rate = 0.0
        instance.min_commission = 0.0
        instance.stamp_tax_rate = 0.0
        instance.transfer_fee_rate = 0.0
        # lot_size 可以是 int (全局统一) 或 Dict[str, int] (按标的设置)
        # 默认 1，这是最通用的设置（适用于美股、加密货币等）。A股回测请务必设置为 100。
        instance.lot_size = 1
        instance._framework_last_session = None
        instance._framework_last_local_date = None
        instance._framework_before_trading_done_date = None
        instance._framework_after_trading_done_date = None
        instance._framework_last_portfolio_state = None
        instance._framework_portfolio_dirty = True
        instance._framework_rejected_order_ids = set()
        instance._framework_stop_flushed = False
        instance._framework_boundary_timers_registered = False
        instance._trading_day_bounds = {}
        instance._oco_groups = {}
        instance._oco_order_to_group = {}
        instance._pending_brackets = {}
        instance._order_group_seq = 0

        return instance

    def __init__(self) -> None:
        """初始化."""
        pass

    def __getstate__(self) -> Dict[str, Any]:
        """
        Pickle 序列化支持.

        排除运行时上下文 (ctx) 和临时对象 (current_bar, current_tick).
        """
        state = self.__dict__.copy()
        if "ctx" in state:
            del state["ctx"]
        if "current_bar" in state:
            del state["current_bar"]
        if "current_tick" in state:
            del state["current_tick"]
        if "_framework_last_session" in state:
            del state["_framework_last_session"]
        if "_framework_last_local_date" in state:
            del state["_framework_last_local_date"]
        if "_framework_before_trading_done_date" in state:
            del state["_framework_before_trading_done_date"]
        if "_framework_after_trading_done_date" in state:
            del state["_framework_after_trading_done_date"]
        if "_framework_last_portfolio_state" in state:
            del state["_framework_last_portfolio_state"]
        if "_framework_portfolio_dirty" in state:
            del state["_framework_portfolio_dirty"]
        if "_framework_rejected_order_ids" in state:
            del state["_framework_rejected_order_ids"]
        if "_framework_stop_flushed" in state:
            del state["_framework_stop_flushed"]
        if "_framework_boundary_timers_registered" in state:
            del state["_framework_boundary_timers_registered"]
        return state

    def __setstate__(self, state: Dict[str, Any]) -> None:
        """Pickle 反序列化支持."""
        self.__dict__.update(state)
        raw_runtime_config = self.__dict__.pop("runtime_config", None)
        if raw_runtime_config is not None:
            self.runtime_config = raw_runtime_config
        elif "_runtime_config" in self.__dict__:
            self.runtime_config = self.__dict__["_runtime_config"]
        else:
            self.runtime_config = StrategyRuntimeConfig(
                enable_precise_day_boundary_hooks=bool(
                    self.__dict__.pop("enable_precise_day_boundary_hooks", False)
                ),
                portfolio_update_eps=float(
                    self.__dict__.pop("portfolio_update_eps", 0.0)
                ),
                error_mode=cast(
                    Literal["raise", "continue", "legacy"],
                    str(self.__dict__.pop("error_mode", "raise")),
                ),
                re_raise_on_error=bool(self.__dict__.pop("re_raise_on_error", True)),
                indicator_mode=cast(
                    Literal["incremental", "precompute"],
                    str(self.__dict__.pop("indicator_mode", "precompute")),
                ),
            )
        self.ctx = None
        self.current_bar = None
        self.current_tick = None
        self._is_restored = True  # 标记为已恢复状态
        self._start_initialized = False
        if not hasattr(self, "_seen_trade_keys"):
            self._seen_trade_keys = set()
        if not hasattr(self, "_seen_trade_key_order"):
            self._seen_trade_key_order = deque()
        if not hasattr(self, "_oco_groups"):
            self._oco_groups = {}
        if not hasattr(self, "_oco_order_to_group"):
            self._oco_order_to_group = {}
        if not hasattr(self, "_pending_brackets"):
            self._pending_brackets = {}
        if not hasattr(self, "_order_group_seq"):
            self._order_group_seq = 0
        _ensure_framework_state_impl(self)

    @property
    def runtime_config(self) -> StrategyRuntimeConfig:
        """返回策略运行时配置."""
        cfg = getattr(self, "_runtime_config", None)
        if isinstance(cfg, StrategyRuntimeConfig):
            return cfg
        if isinstance(cfg, dict):
            self._runtime_config = StrategyRuntimeConfig(**cfg)
            return self._runtime_config
        self._runtime_config = StrategyRuntimeConfig()
        return self._runtime_config

    @runtime_config.setter
    def runtime_config(
        self, value: Union[StrategyRuntimeConfig, Dict[str, Any]]
    ) -> None:
        """设置策略运行时配置."""
        if isinstance(value, StrategyRuntimeConfig):
            self._runtime_config = StrategyRuntimeConfig(
                enable_precise_day_boundary_hooks=value.enable_precise_day_boundary_hooks,
                portfolio_update_eps=value.portfolio_update_eps,
                error_mode=value.error_mode,
                re_raise_on_error=value.re_raise_on_error,
                indicator_mode=value.indicator_mode,
            )
            return
        if isinstance(value, dict):
            self._runtime_config = StrategyRuntimeConfig(**value)
            return
        raise TypeError(
            "runtime_config must be StrategyRuntimeConfig or Dict[str, Any]"
        )

    @property
    def enable_precise_day_boundary_hooks(self) -> bool:
        """是否启用边界定时器精确交易日钩子."""
        return self.runtime_config.enable_precise_day_boundary_hooks

    @enable_precise_day_boundary_hooks.setter
    def enable_precise_day_boundary_hooks(self, value: bool) -> None:
        """设置边界定时器精确交易日钩子开关."""
        cfg = self.runtime_config
        self.runtime_config = StrategyRuntimeConfig(
            enable_precise_day_boundary_hooks=bool(value),
            portfolio_update_eps=cfg.portfolio_update_eps,
            error_mode=cfg.error_mode,
            re_raise_on_error=cfg.re_raise_on_error,
            indicator_mode=cfg.indicator_mode,
        )

    @property
    def portfolio_update_eps(self) -> float:
        """返回账户快照更新阈值."""
        return self.runtime_config.portfolio_update_eps

    @portfolio_update_eps.setter
    def portfolio_update_eps(self, value: float) -> None:
        """设置账户快照更新阈值."""
        cfg = self.runtime_config
        self.runtime_config = StrategyRuntimeConfig(
            enable_precise_day_boundary_hooks=cfg.enable_precise_day_boundary_hooks,
            portfolio_update_eps=float(value),
            error_mode=cfg.error_mode,
            re_raise_on_error=cfg.re_raise_on_error,
            indicator_mode=cfg.indicator_mode,
        )

    @property
    def error_mode(self) -> str:
        """返回错误处理模式."""
        return self.runtime_config.error_mode

    @error_mode.setter
    def error_mode(self, value: str) -> None:
        """设置错误处理模式."""
        cfg = self.runtime_config
        self.runtime_config = StrategyRuntimeConfig(
            enable_precise_day_boundary_hooks=cfg.enable_precise_day_boundary_hooks,
            portfolio_update_eps=cfg.portfolio_update_eps,
            error_mode=cast(Literal["raise", "continue", "legacy"], value),
            re_raise_on_error=cfg.re_raise_on_error,
            indicator_mode=cfg.indicator_mode,
        )

    @property
    def re_raise_on_error(self) -> bool:
        """返回是否在 on_error 后继续抛出异常."""
        return self.runtime_config.re_raise_on_error

    @re_raise_on_error.setter
    def re_raise_on_error(self, value: bool) -> None:
        """设置 on_error 后是否继续抛出异常."""
        cfg = self.runtime_config
        self.runtime_config = StrategyRuntimeConfig(
            enable_precise_day_boundary_hooks=cfg.enable_precise_day_boundary_hooks,
            portfolio_update_eps=cfg.portfolio_update_eps,
            error_mode=cfg.error_mode,
            re_raise_on_error=bool(value),
            indicator_mode=cfg.indicator_mode,
        )

    @property
    def indicator_mode(self) -> Literal["incremental", "precompute"]:
        """返回指标执行模式."""
        return self.runtime_config.indicator_mode

    @indicator_mode.setter
    def indicator_mode(self, value: Literal["incremental", "precompute"]) -> None:
        cfg = self.runtime_config
        self.runtime_config = StrategyRuntimeConfig(
            enable_precise_day_boundary_hooks=cfg.enable_precise_day_boundary_hooks,
            portfolio_update_eps=cfg.portfolio_update_eps,
            error_mode=cfg.error_mode,
            re_raise_on_error=cfg.re_raise_on_error,
            indicator_mode=value,
        )

    @property
    def is_restored(self) -> bool:
        """检查策略是否是从快照恢复的."""
        return getattr(self, "_is_restored", False)

    def on_start(self) -> None:
        """
        策略启动时调用.

        Warm Start 注意：如果策略是从快照恢复的，此方法仍会被调用。
        如果在此处初始化指标，请务必检查属性是否存在 (hasattr)，
        或者使用 self.is_restored 属性判断，避免覆盖已恢复的状态。
        """
        pass

    def on_resume(self) -> None:
        """
        策略从快照恢复时调用 (Warm Start).

        仅在 Warm Start 模式下调用，且在 on_start 之前调用。
        用于重新建立非持久化连接、加载外部资源等。
        """
        pass

    def _on_start_internal(self) -> None:
        """内部启动回调."""
        if self._start_initialized:
            return

        # 如果是恢复模式，先调用 on_resume
        if self.is_restored:
            self.on_resume()

        self.on_start()
        self._start_initialized = True

    def on_stop(self) -> None:
        """
        策略停止时调用.

        在此处进行资源清理或结果统计.
        """
        pass

    def _on_stop_internal(self) -> None:
        """内部停止回调，用于补发框架级结束钩子."""
        _ensure_framework_state_impl(self)
        _dispatch_shutdown_hooks_impl(self)
        _call_user_callback_impl(self, "on_stop")

    def log(self, msg: str, level: int = logging.INFO) -> None:
        """
        输出日志 (自动添加当前回测时间).

        :param msg: 日志内容
        :param level: 日志等级 (logging.INFO, logging.WARNING, etc.)
        """
        _log_impl(self, msg, level)

    @property
    def symbol(self) -> str:
        """获取当前正在处理的标的代码 (Proxy to current_bar/tick)."""
        return self._resolve_symbol(None)

    @property
    def close(self) -> float:
        """获取当前最新价 (Close 或 LastPrice)."""
        if self.current_bar:
            return self.current_bar.close
        elif self.current_tick:
            return self.current_tick.price
        return 0.0

    @property
    def open(self) -> float:
        """获取当前开盘价 (仅 Bar 模式有效)."""
        if self.current_bar:
            return self.current_bar.open
        return 0.0

    @property
    def high(self) -> float:
        """获取当前最高价 (仅 Bar 模式有效)."""
        if self.current_bar:
            return self.current_bar.high
        return 0.0

    @property
    def low(self) -> float:
        """获取当前最低价 (仅 Bar 模式有效)."""
        if self.current_bar:
            return self.current_bar.low
        return 0.0

    @property
    def volume(self) -> float:
        """获取当前成交量."""
        if self.current_bar:
            return self.current_bar.volume
        elif self.current_tick:
            return self.current_tick.volume
        return 0.0

    def schedule(
        self, trigger_time: Union[str, dt.datetime, pd.Timestamp], payload: str
    ) -> None:
        """
        注册单次定时任务 (Simplified).

        :param trigger_time: 触发时间 (支持 "2023-01-01 14:55:00", datetime, Timestamp)
        :param payload: 回调标识
        """
        _schedule_impl(self, trigger_time, payload)

    def add_daily_timer(self, time_str: str, payload: str) -> None:
        """
        注册每日定时任务 (Daily Timer).

        :param time_str: 时间字符串 (例如 "14:55:00")
        :param payload: 回调标识
        """
        _add_daily_timer_impl(self, time_str, payload)

    def to_local_time(self, timestamp: int) -> pd.Timestamp:
        """
        将 UTC 纳秒时间戳转换为本地时间 (Timestamp).

        :param timestamp: UTC 纳秒时间戳 (int64)
        :return: 本地时间 (pd.Timestamp)
        """
        return _to_local_time_impl(self, timestamp)

    def format_time(self, timestamp: int, fmt: str = "%Y-%m-%d %H:%M:%S") -> str:
        """
        将 UTC 纳秒时间戳格式化为本地时间字符串.

        :param timestamp: UTC 纳秒时间戳 (int64)
        :param fmt: 时间格式字符串
        :return: 格式化后的时间字符串
        """
        return _format_time_impl(self, timestamp, fmt)

    @property
    def now(self) -> Optional[pd.Timestamp]:
        """
        获取当前回测时间的本地时间表示.

        如果当前没有 Bar 或 Tick，则返回 None.
        """
        return _now_impl(self)

    def set_history_depth(self, depth: int) -> None:
        """
        设置历史数据回溯长度.

        :param depth: 保留的 Bar 数量 (0 表示不保留)
        """
        _set_history_depth_impl(self, depth)

    def set_rolling_window(self, train_window: int, step: int) -> None:
        """
        设置滚动训练窗口参数.

        :param train_window: 训练数据长度 (Bars)
        :param step: 滚动步长 (每隔多少个 Bar 触发一次训练)
        """
        _set_rolling_window_impl(self, train_window, step)

    def get_history(
        self, count: int, symbol: Optional[str] = None, field: str = "close"
    ) -> np.ndarray:
        """
        获取历史数据 (类似 Zipline data.history).

        :param count: 获取的数据长度 (必须 <= history_depth)
        :param symbol: 标的代码 (默认当前 Bar 的 symbol)
        :param field: 字段名 (open, high, low, close, volume)
        :return: Numpy 数组
        """
        return _get_history_impl(self, count, symbol, field)

    def get_history_df(self, count: int, symbol: Optional[str] = None) -> pd.DataFrame:
        """
        获取历史数据 DataFrame (Open, High, Low, Close, Volume).

        :param count: 数据长度
        :param symbol: 标的代码
        :return: pd.DataFrame
        """
        return _get_history_df_impl(self, count, symbol)

    def get_rolling_data(
        self, length: Optional[int] = None, symbol: Optional[str] = None
    ) -> tuple[pd.DataFrame, Optional[pd.Series]]:
        """
        获取滚动训练数据.

        :param length: 数据长度 (默认使用 set_rolling_window 设置的 train_window)
        :param symbol: 标的代码
        :return: (X, y) 默认为 (DataFrame, None)
        """
        return _get_rolling_data_impl(self, length, symbol)

    def on_train_signal(self, context: Any) -> None:
        """
        滚动训练信号回调.

        默认实现：如果配置了 self.model，则自动执行数据准备和训练.

        :param context: 策略上下文 (通常是 self)
        """
        _on_train_signal_impl(self, context)

    def prepare_features(
        self, df: pd.DataFrame, mode: str = "training"
    ) -> Tuple[Any, Any]:
        """
        Prepare features and labels for ML model.

        Must be implemented by user if using auto-training.

        :param df: Raw dataframe from get_rolling_data
        :param mode: "training" or "inference".
                     If "training", return (X, y).
                     If "inference", return (X_last_row, None) or just X.
        :return: (X, y)
        """
        raise NotImplementedError(
            "You must implement prepare_features(self, df, mode) for auto-training"
        )

    def _auto_configure_model(self) -> None:
        """Apply model validation configuration if present."""
        _auto_configure_model_impl(self)

    def set_sizer(self, sizer: Sizer) -> None:
        """设置仓位管理器."""
        self.sizer = sizer

    def register_indicator(self, name: str, indicator: "Indicator") -> None:
        """
        Register an indicator.

        This allows accessing the indicator via self.name and ensures it is
        calculated before the backtest starts.
        """
        self.register_precomputed_indicator(name, indicator)

    def register_precomputed_indicator(self, name: str, indicator: "Indicator") -> None:
        """注册预计算指标."""
        if self.indicator_mode != "precompute":
            raise ValueError(
                "register_precomputed_indicator requires indicator_mode='precompute'"
            )
        if indicator not in self._precomputed_indicators:
            self._precomputed_indicators.append(indicator)
        setattr(self, name, indicator)

    def register_incremental_indicator(
        self,
        name: str,
        indicator: Any,
        source: str = "close",
        symbols: Optional[Union[str, List[str], Tuple[str, ...], set[str]]] = None,
    ) -> None:
        """注册增量指标."""
        if self.indicator_mode != "incremental":
            raise ValueError(
                "register_incremental_indicator requires indicator_mode='incremental'"
            )
        source_key = str(source).strip().lower()
        if source_key not in {"open", "high", "low", "close", "volume"}:
            raise ValueError("source must be one of: open, high, low, close, volume")
        symbol_filter = self._normalize_indicator_symbols(symbols)
        self._incremental_indicators[name] = {
            "indicator": indicator,
            "source": source_key,
            "symbols": symbol_filter,
        }
        setattr(self, name, indicator)

    def subscribe(self, instrument_id: str) -> None:
        """
        Subscribe to market data for an instrument.

        :param instrument_id: The instrument identifier (e.g., '600000').
        """
        if instrument_id not in self._subscriptions:
            self._subscriptions.append(instrument_id)

    def _prepare_indicators(self, data: Dict[str, pd.DataFrame]) -> None:
        """Pre-calculate indicators for precompute mode."""
        if self.indicator_mode != "precompute":
            return
        if not self._precomputed_indicators:
            return

        for ind in self._precomputed_indicators:
            for sym, df in data.items():
                # Calculate and cache inside indicator
                ind(df, sym)

    def _update_incremental_indicators(self, bar: Bar) -> None:
        if self.indicator_mode != "incremental":
            return
        if not self._incremental_indicators:
            return
        for item in self._incremental_indicators.values():
            symbol_filter = item["symbols"]
            if symbol_filter is not None and bar.symbol not in symbol_filter:
                continue
            ind = item["indicator"]
            source = item["source"]
            value = getattr(bar, source)
            ind.update(value)

    def _normalize_indicator_symbols(
        self,
        symbols: Optional[Union[str, List[str], Tuple[str, ...], set[str]]],
    ) -> Optional[set[str]]:
        if symbols is None:
            return None
        if isinstance(symbols, str):
            symbol_text = symbols.strip()
            if not symbol_text:
                raise ValueError("symbols cannot contain empty symbol")
            return {symbol_text}
        if isinstance(symbols, (list, tuple, set)):
            normalized: set[str] = set()
            for symbol in symbols:
                symbol_text = str(symbol).strip()
                if not symbol_text:
                    raise ValueError("symbols cannot contain empty symbol")
                normalized.add(symbol_text)
            if not normalized:
                raise ValueError("symbols cannot be empty")
            return normalized
        raise TypeError("symbols must be str, list[str], tuple[str, ...], set[str]")

    def on_order(self, order: Any) -> None:
        """
        订单状态更新回调.

        Args:
            order: 订单对象
        """
        pass

    def on_trade(self, trade: Any) -> None:
        """
        成交回调.

        Args:
            trade: 成交对象
        """
        pass

    def on_session_start(self, session: Any, timestamp: int) -> None:
        """会话开始回调."""
        pass

    def on_session_end(self, session: Any, timestamp: int) -> None:
        """会话结束回调."""
        pass

    def before_trading(self, trading_date: dt.date, timestamp: int) -> None:
        """交易日开始前回调."""
        pass

    def after_trading(self, trading_date: dt.date, timestamp: int) -> None:
        """交易日结束后回调."""
        pass

    def on_reject(self, order: Any) -> None:
        """拒单回调."""
        pass

    def on_portfolio_update(self, snapshot: Dict[str, Any]) -> None:
        """账户变化回调."""
        pass

    def on_error(self, error: Exception, source: str, payload: Any = None) -> None:
        """错误回调."""
        pass

    def _check_order_events(self) -> None:
        _check_order_events_impl(self)

    def _trade_event_key(self, trade: Any) -> Tuple[Any, ...]:
        return _trade_event_key_impl(self, trade)

    def _key_value(self, value: Any) -> Any:
        return _key_value_impl(value)

    def _remember_trade_key(self, key: Tuple[Any, ...]) -> bool:
        return _remember_trade_key_impl(self, key)

    def _trade_dedupe_cache_limit(self) -> int:
        return _trade_dedupe_cache_limit_impl(self)

    def _on_bar_event(self, bar: Bar, ctx: StrategyContext) -> None:
        _on_bar_event_impl(self, bar, ctx)

    def _on_tick_event(self, tick: Tick, ctx: StrategyContext) -> None:
        _on_tick_event_impl(self, tick, ctx)

    def _on_timer_event(self, payload: str, ctx: StrategyContext) -> None:
        _on_timer_event_impl(self, payload, ctx)

    def on_bar(self, bar: Bar) -> None:
        """
        策略逻辑入口 (Bar 数据).

        用户应重写此方法.
        """
        pass

    @property
    def position(self) -> Position:
        """
        获取当前处理标的的持仓对象.

        支持常见的策略编写语法:
        if self.position.size == 0:
            ...
        """
        if self.ctx is None:
            raise RuntimeError("Context not ready")

        symbol = self._resolve_symbol(None)
        return Position(self.ctx, symbol)

    def on_tick(self, tick: Tick) -> None:
        """
        策略逻辑入口 (Tick 数据).

        用户应重写此方法.
        """
        pass

    def on_timer(self, payload: str) -> None:
        """
        策略逻辑入口 (Timer 事件).

        用户应重写此方法.
        """
        pass

    def _resolve_symbol(self, symbol: Optional[str] = None) -> str:
        return _resolve_symbol_impl(self, symbol)

    def get_position(self, symbol: Optional[str] = None) -> float:
        """
        获取指定标的的持仓数量.

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)

        Returns:
            float: 持仓数量 (正数为多头, 负数为空头)
        """
        return _get_position_impl(self, symbol)

    def get_available_position(self, symbol: Optional[str] = None) -> float:
        """
        获取指定标的的可用持仓数量 (考虑 T+1 等限制).

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)

        Returns:
            float: 可用持仓数量
        """
        return _get_available_position_impl(self, symbol)

    def hold_bar(self, symbol: Optional[str] = None) -> int:
        """
        获取当前持仓持有的 Bar 数量.

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)

        Returns:
            int: 持有的 Bar 数量. 如果未持仓，返回 0.
        """
        return _hold_bar_impl(self, symbol)

    def get_positions(self) -> Dict[str, float]:
        """
        获取所有持仓信息.

        Returns:
            Dict[str, float]: 持仓字典 {symbol: quantity}
        """
        return _get_positions_impl(self)

    def get_open_orders(self, symbol: Optional[str] = None) -> List[Any]:
        """
        获取当前未完成的订单.

        Args:
            symbol: 标的代码 (如果为 None，返回所有标的订单)

        Returns:
            List[Order]: 订单列表
        """
        return _get_open_orders_impl(self, symbol)

    def get_order(self, order_id: str) -> Optional[Any]:
        """
        获取指定订单详情.

        Args:
            order_id: 订单 ID

        Returns:
            Order: 订单对象，如果未找到则返回 None
        """
        return _get_order_impl(self, order_id)

    def get_account(self) -> Dict[str, float]:
        """
        获取账户资金详情快照.

        Returns:
            Dict: 包含以下字段:
                - cash: 可用资金
                - equity: 总权益 (现金 + 持仓市值)
                - market_value: 持仓总市值 (equity - cash)
                - frozen_cash: (预留) 冻结资金, 暂为 0.0
                - margin: (预留) 占用保证金, 暂为 0.0
        """
        return _get_account_impl(self)

    def get_trades(self) -> List[Any]:
        """
        获取所有历史成交记录 (Closed Trades).

        Returns:
            List[ClosedTrade]: 已平仓交易列表
        """
        if self.ctx:
            return self.ctx.closed_trades
        return []

    def cancel_order(self, order_id: str) -> None:
        """
        取消指定订单.

        Args:
            order_id: 订单 ID
        """
        _cancel_order_impl(self, order_id)

    def cancel_all_orders(self, symbol: Optional[str] = None) -> None:
        """
        取消当前所有未完成的订单.

        Args:
            symbol: 标的代码 (如果不填, 取消所有标的订单)
        """
        _cancel_all_orders_impl(self, symbol)

    def create_oco_order_group(
        self,
        first_order_id: str,
        second_order_id: str,
        group_id: Optional[str] = None,
    ) -> str:
        """
        创建 OCO 订单组.

        当组内任一订单成交时，自动撤销另一订单。
        """
        first = first_order_id.strip()
        second = second_order_id.strip()
        if not first or not second:
            raise ValueError("OCO order ids cannot be empty")
        if first == second:
            raise ValueError("OCO order ids must be different")

        if group_id is None or not str(group_id).strip():
            self._order_group_seq += 1
            group_id = f"oco-{self._order_group_seq}"
        group_key = str(group_id).strip()

        self._detach_oco_order(first)
        self._detach_oco_order(second)

        self._oco_groups[group_key] = {first, second}
        self._oco_order_to_group[first] = group_key
        self._oco_order_to_group[second] = group_key
        return group_key

    def place_bracket_order(
        self,
        symbol: str,
        quantity: float,
        entry_price: Optional[float] = None,
        stop_trigger_price: Optional[float] = None,
        take_profit_price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        entry_tag: Optional[str] = None,
        stop_tag: Optional[str] = None,
        take_profit_tag: Optional[str] = None,
    ) -> str:
        """
        创建 Bracket 订单.

        先提交进场单，待进场成交后自动挂出止损/止盈，并绑定 OCO。
        """
        if quantity <= 0:
            raise ValueError("quantity must be > 0")
        if stop_trigger_price is None and take_profit_price is None:
            raise ValueError("stop_trigger_price or take_profit_price must be provided")

        entry_order_id = self.buy(
            symbol=symbol,
            quantity=quantity,
            price=entry_price,
            time_in_force=time_in_force,
            tag=entry_tag,
        )
        if not entry_order_id:
            raise RuntimeError("failed to submit bracket entry order")

        self._pending_brackets[entry_order_id] = {
            "symbol": symbol,
            "quantity": float(quantity),
            "stop_trigger_price": stop_trigger_price,
            "take_profit_price": take_profit_price,
            "time_in_force": time_in_force,
            "stop_tag": stop_tag,
            "take_profit_tag": take_profit_tag,
        }
        return entry_order_id

    def _process_order_groups(self, trade: Any) -> None:
        self._process_pending_bracket(trade)
        self._process_oco_trade(trade)

    def _process_pending_bracket(self, trade: Any) -> None:
        order_id = str(getattr(trade, "order_id", "") or "")
        if not order_id:
            return

        bracket = self._pending_brackets.pop(order_id, None)
        if bracket is None:
            return

        trade_symbol = getattr(trade, "symbol", None)
        symbol = str(trade_symbol) if trade_symbol else str(bracket["symbol"])

        trade_qty = getattr(trade, "quantity", None)
        if trade_qty is None:
            quantity = float(bracket["quantity"])
        else:
            quantity = float(trade_qty)

        stop_order_id = ""
        take_order_id = ""
        stop_trigger_price = bracket["stop_trigger_price"]
        take_profit_price = bracket["take_profit_price"]
        time_in_force = cast(Optional[TimeInForce], bracket["time_in_force"])

        if stop_trigger_price is not None:
            stop_order_id = self.sell(
                symbol=symbol,
                quantity=quantity,
                trigger_price=float(stop_trigger_price),
                time_in_force=time_in_force,
                tag=cast(Optional[str], bracket["stop_tag"]),
            )

        if take_profit_price is not None:
            take_order_id = self.sell(
                symbol=symbol,
                quantity=quantity,
                price=float(take_profit_price),
                time_in_force=time_in_force,
                tag=cast(Optional[str], bracket["take_profit_tag"]),
            )

        if stop_order_id and take_order_id:
            self.create_oco_order_group(stop_order_id, take_order_id)

    def _process_oco_trade(self, trade: Any) -> None:
        order_id = str(getattr(trade, "order_id", "") or "")
        if not order_id:
            return

        group_id = self._oco_order_to_group.get(order_id)
        if not group_id:
            return

        group_orders = self._oco_groups.get(group_id, set())
        peer_orders = [oid for oid in group_orders if oid != order_id]
        for peer_order_id in peer_orders:
            self.cancel_order(peer_order_id)

        self._remove_oco_group(group_id)

    def _detach_oco_order(self, order_id: str) -> None:
        old_group = self._oco_order_to_group.get(order_id)
        if not old_group:
            return
        group_orders = self._oco_groups.get(old_group)
        if group_orders is not None:
            group_orders.discard(order_id)
            if len(group_orders) <= 1:
                self._remove_oco_group(old_group)
        self._oco_order_to_group.pop(order_id, None)

    def _remove_oco_group(self, group_id: str) -> None:
        group_orders = self._oco_groups.pop(group_id, set())
        for oid in group_orders:
            self._oco_order_to_group.pop(oid, None)

    def buy(
        self,
        symbol: Optional[str] = None,
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        trigger_price: Optional[float] = None,
        tag: Optional[str] = None,
    ) -> str:
        """
        买入下单.

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)
            quantity: 数量 (如果不填, 使用 Sizer 计算)
            price: 限价 (None 为市价)
            time_in_force: 订单有效期
            trigger_price: 触发价 (止损/止盈)
            tag: 订单标签

        Returns:
            str: 订单 ID
        """
        return _buy_impl(
            self, symbol, quantity, price, time_in_force, trigger_price, tag
        )

    def sell(
        self,
        symbol: Optional[str] = None,
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        trigger_price: Optional[float] = None,
        tag: Optional[str] = None,
    ) -> str:
        """
        卖出下单.

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)
            quantity: 数量 (如果不填, 默认卖出当前标的所有持仓)
            price: 限价 (None 为市价)
            time_in_force: 订单有效期
            trigger_price: 触发价 (止损/止盈)
            tag: 订单标签

        Returns:
            str: 订单 ID
        """
        return _sell_impl(
            self, symbol, quantity, price, time_in_force, trigger_price, tag
        )

    def submit_order(
        self,
        symbol: Optional[str] = None,
        side: str = "Buy",
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce | str] = None,
        trigger_price: Optional[float] = None,
        tag: Optional[str] = None,
        client_order_id: Optional[str] = None,
        order_type: Optional[str] = None,
        extra: Optional[Dict[str, Any]] = None,
        trail_offset: Optional[float] = None,
        trail_reference_price: Optional[float] = None,
    ) -> str:
        """
        统一下单接口.

        该接口在回测与实盘模式均可调用，实盘模式下会由 LiveRunner 注入增强能力。
        """
        return _submit_order_impl(
            self,
            symbol=symbol,
            side=side,
            quantity=quantity,
            price=price,
            time_in_force=time_in_force,
            trigger_price=trigger_price,
            tag=tag,
            client_order_id=client_order_id,
            order_type=order_type,
            extra=extra,
            trail_offset=trail_offset,
            trail_reference_price=trail_reference_price,
        )

    def can_submit_client_order(self, client_order_id: str) -> bool:
        """
        检查 client_order_id 是否可再次提交.

        该方法在 LiveRunner 的 broker_live 模式下会被注入真实实现。
        """
        _ = client_order_id
        return True

    def get_execution_capabilities(self) -> Dict[str, Any]:
        """获取当前执行能力描述."""
        return _get_execution_capabilities_impl(self)

    def stop_buy(
        self,
        symbol: Optional[str] = None,
        trigger_price: float = 0.0,
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
    ) -> None:
        """
        发送止损买入单 (Stop Buy Order).

        当市价上涨突破 trigger_price 时触发买入.
        - 如果 price 为 None, 触发后转为市价单 (Stop Market).
        - 如果 price 不为 None, 触发后转为限价单 (Stop Limit).
        """
        _stop_buy_impl(self, symbol, trigger_price, quantity, price, time_in_force)

    def stop_sell(
        self,
        symbol: Optional[str] = None,
        trigger_price: float = 0.0,
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
    ) -> None:
        """
        发送止损卖出单 (Stop Sell Order).

        当市价下跌跌破 trigger_price 时触发卖出.
        - 如果 price 为 None, 触发后转为市价单 (Stop Market).
        - 如果 price 不为 None, 触发后转为限价单 (Stop Limit).
        """
        _stop_sell_impl(self, symbol, trigger_price, quantity, price, time_in_force)

    def place_trailing_stop(
        self,
        symbol: str,
        quantity: float,
        trail_offset: float,
        side: str = "Sell",
        trail_reference_price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        tag: Optional[str] = None,
    ) -> str:
        """创建跟踪止损单 (StopTrail)."""
        if quantity <= 0:
            raise ValueError("quantity must be > 0")
        if trail_offset <= 0:
            raise ValueError("trail_offset must be > 0")
        return self.submit_order(
            symbol=symbol,
            side=side,
            quantity=quantity,
            time_in_force=time_in_force,
            tag=tag,
            order_type="StopTrail",
            trail_offset=trail_offset,
            trail_reference_price=trail_reference_price,
        )

    def place_trailing_stop_limit(
        self,
        symbol: str,
        quantity: float,
        price: float,
        trail_offset: float,
        side: str = "Sell",
        trail_reference_price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        tag: Optional[str] = None,
    ) -> str:
        """创建跟踪止损限价单 (StopTrailLimit)."""
        if quantity <= 0:
            raise ValueError("quantity must be > 0")
        if price <= 0:
            raise ValueError("price must be > 0")
        if trail_offset <= 0:
            raise ValueError("trail_offset must be > 0")
        return self.submit_order(
            symbol=symbol,
            side=side,
            quantity=quantity,
            price=price,
            time_in_force=time_in_force,
            tag=tag,
            order_type="StopTrailLimit",
            trail_offset=trail_offset,
            trail_reference_price=trail_reference_price,
        )

    def get_portfolio_value(self) -> float:
        """计算当前投资组合总价值 (现金 + 持仓市值)."""
        return _get_portfolio_value_impl(self)

    @property
    def equity(self) -> float:
        """
        获取当前账户总权益 (现金 + 持仓市值).

        等同于 get_portfolio_value().
        """
        return self.get_portfolio_value()

    def order_target(
        self,
        target: float,
        symbol: Optional[str] = None,
        price: Optional[float] = None,
        **kwargs: Any,
    ) -> None:
        """
        调整仓位到目标数量.

        :param target: 目标持仓数量 (例如 100, -100)
        :param symbol: 标的代码
        :param price: 限价 (可选)
        :param kwargs: 其他下单参数
        """
        _order_target_impl(self, target, symbol, price, **kwargs)

    def _calculate_max_buy_qty(self, symbol: str, price: float, cash: float) -> float:
        """
        计算考虑费率后的最大可买数量.

        :param symbol: 标的代码
        :param price: 交易价格
        :param cash: 可用资金
        :return: 最大可买数量
        """
        return _calculate_max_buy_qty_impl(self, symbol, price, cash)

    def order_target_value(
        self,
        target_value: float,
        symbol: Optional[str] = None,
        price: Optional[float] = None,
        **kwargs: Any,
    ) -> None:
        """
        调整仓位到目标价值.

        :param target_value: 目标持仓价值
        :param symbol: 标的代码
        :param price: 限价 (可选)
        :param kwargs: 其他下单参数
        """
        _order_target_value_impl(self, target_value, symbol, price, **kwargs)

    def order_target_percent(
        self,
        target_percent: float,
        symbol: Optional[str] = None,
        price: Optional[float] = None,
        **kwargs: Any,
    ) -> None:
        """
        调整仓位到目标百分比.

        :param target_percent: 目标持仓比例 (0.5 = 50%)
        :param symbol: 标的代码
        :param price: 限价 (可选)
        :param kwargs: 其他下单参数
        """
        _order_target_percent_impl(self, target_percent, symbol, price, **kwargs)

    def order_target_weights(
        self,
        target_weights: Dict[str, float],
        price_map: Optional[Dict[str, float]] = None,
        liquidate_unmentioned: bool = False,
        allow_leverage: bool = False,
        rebalance_tolerance: float = 0.0,
        **kwargs: Any,
    ) -> None:
        """
        按多标的目标权重调仓.

        :param target_weights: 目标权重字典 {symbol: weight}
        :param price_map: 每个标的的委托价格字典（可选）
        :param liquidate_unmentioned: 是否清仓未出现在目标权重中的现有持仓
        :param allow_leverage: 是否允许目标权重和超过 1.0
        :param rebalance_tolerance: 调仓容忍阈值（按组合市值比例）
        :param kwargs: 其他下单参数
        """
        _order_target_weights_impl(
            self,
            target_weights,
            price_map,
            liquidate_unmentioned,
            allow_leverage,
            rebalance_tolerance,
            **kwargs,
        )

    def buy_all(self, symbol: Optional[str] = None) -> None:
        """
        全仓买入 (Buy All).

        使用当前所有可用资金买入.

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)
        """
        _buy_all_impl(self, symbol)

    def close_position(self, symbol: Optional[str] = None) -> None:
        """
        平仓 (Close Position).

        卖出/买入以抵消当前持仓.

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)
        """
        _close_position_impl(self, symbol)

    def short(
        self,
        symbol: Optional[str] = None,
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        trigger_price: Optional[float] = None,
    ) -> None:
        """
        卖出开空 (Short Sell).

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)
            quantity: 数量 (如果不填, 使用 Sizer 计算)
            price: 限价 (None 为市价)
            time_in_force: 订单有效期
            trigger_price: 触发价 (止损/止盈)
        """
        _short_impl(self, symbol, quantity, price, time_in_force, trigger_price)

    def cover(
        self,
        symbol: Optional[str] = None,
        quantity: Optional[float] = None,
        price: Optional[float] = None,
        time_in_force: Optional[TimeInForce] = None,
        trigger_price: Optional[float] = None,
    ) -> None:
        """
        买入平空 (Buy to Cover).

        Args:
            symbol: 标的代码 (如果不填, 默认使用当前 Bar/Tick 的 symbol)
            quantity: 数量 (如果不填, 默认平掉当前标的所有空头持仓)
            price: 限价 (None 为市价)
            time_in_force: 订单有效期
            trigger_price: 触发价 (止损/止盈)
        """
        _cover_impl(self, symbol, quantity, price, time_in_force, trigger_price)

    def get_cash(self) -> float:
        """获取现金."""
        return _get_cash_impl(self)


class VectorizedStrategy(Strategy):
    """
    向量化策略基类 (Vectorized Strategy Base Class).

    支持预计算指标的高速回测模式.
    用户应在回测前使用 Pandas/Numpy 计算好所有指标,
    然后通过本类提供的高速游标访问机制在 on_bar 中读取.
    """

    def __init__(self, precalculated_data: Dict[str, Dict[str, np.ndarray]]) -> None:
        """
        Initialize VectorizedStrategy.

        :param precalculated_data: 预计算数据字典
                                  Structure: {symbol: {indicator_name: numpy_array}}
        """
        super().__init__()
        self.precalc = precalculated_data
        # 游标管理: {symbol: index}
        self.cursors: defaultdict[str, int] = defaultdict(int)

        # 默认禁用 Python 侧历史数据缓存以提升性能
        self.set_history_depth(0)

    def _on_bar_event(self, bar: Bar, ctx: StrategyContext) -> None:
        """Wrap the user on_bar handler internally."""
        super()._on_bar_event(bar, ctx)
        self.cursors[bar.symbol] += 1

    def get_value(self, name: str, symbol: Optional[str] = None) -> float:
        """
        获取当前 Bar 对应的预计算指标值.

        Args:
            name: 指标名称
            symbol: 标的代码 (如果不填, 默认使用当前 Bar 的 symbol)

        Returns:
            指标值 (float). 如果不存在或越界，返回 nan.
        """
        symbol = self._resolve_symbol(symbol)
        idx = self.cursors[symbol]

        try:
            return float(self.precalc[symbol][name][idx])
        except (KeyError, IndexError):
            return float("nan")



import logging
from typing import List, Optional

import polars as pl

from ..data import ParquetDataCatalog
from .parser import ExpressionParser

logger = logging.getLogger(__name__)


class FactorEngine:
    """Engine for calculating factors using Polars Lazy API."""

    def __init__(self, catalog: ParquetDataCatalog):
        """Initialize the FactorEngine with a data catalog."""
        self.catalog = catalog
        self.parser = ExpressionParser()

    def load_data(self) -> pl.LazyFrame:
        """
        Load all data from catalog as a LazyFrame.

        Assumes data is partitioned by symbol or simply stored in separate files.
        """
        # We look for all parquet files under the root
        # Pattern: root/**/data.parquet
        # This covers:
        # - root/AAPL/data.parquet (ParquetDataCatalog default)
        # - root/symbol=AAPL/data.parquet (Hive style)
        pattern = str(self.catalog.root / "**" / "*.parquet")

        try:
            # use_pyarrow=True often helps with nested paths compatibility
            lf = pl.scan_parquet(pattern)
            return lf
        except Exception as e:
            logger.warning(f"Failed to scan parquet files: {e}")
            return pl.DataFrame().lazy()

    def _ensure_date_column(self, lf: pl.LazyFrame) -> pl.LazyFrame:
        """Ensure 'date' column exists, renaming from common variants if needed."""
        # Check schema to find date column
        # Note: collect_schema() is preferred in newer Polars, .schema in older
        try:
            schema = lf.collect_schema()
        except AttributeError:
            schema = lf.schema  # type: ignore

        cols = schema.names() if hasattr(schema, "names") else list(schema.keys())
        logger.info(f"Detected columns: {cols}")

        if "date" in cols:
            return lf
        elif "index" in cols:
            return lf.rename({"index": "date"})
        elif "datetime" in cols:
            return lf.rename({"datetime": "date"})
        elif "__index_level_0__" in cols:
            return lf.rename({"__index_level_0__": "date"})
        else:
            # Fallback: assume user knows what they are doing or it will fail later
            logger.warning(
                f"Could not identify 'date' column from {cols}. "
                "Expecting 'date', 'index', 'datetime', or '__index_level_0__'."
            )
            return lf

    def run(
        self,
        expr_str: str,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
    ) -> pl.DataFrame:
        """
        Execute a factor expression.

        :param expr_str: The factor expression string (e.g., "Rank(Ts_Mean(Close, 5))")
        :param start_date: Filter start date (inclusive).
        :param end_date: Filter end date (inclusive).
        :return: DataFrame with [date, symbol, factor] columns.
        """
        lf = self.load_data()
        return self.run_on_data(lf, expr_str, start_date, end_date)

    def run_on_data(
        self,
        data: pl.LazyFrame | pl.DataFrame,
        expr_str: str,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
    ) -> pl.DataFrame:
        """
        Execute a factor expression on provided data.

        :param data: Polars DataFrame or LazyFrame containing input data.
        :param expr_str: The factor expression string.
        :param start_date: Filter start date (inclusive).
        :param end_date: Filter end date (inclusive).
        :return: DataFrame with [date, symbol, factor] columns.
        """
        if isinstance(data, pl.DataFrame):
            lf = data.lazy()
        else:
            lf = data

        lf = self._ensure_date_column(lf)
        lf = lf.sort(["symbol", "date"])

        if start_date:
            lf = lf.filter(
                pl.col("date") >= pl.lit(start_date).str.to_datetime(strict=False)
            )
        if end_date:
            lf = lf.filter(
                pl.col("date") <= pl.lit(end_date).str.to_datetime(strict=False)
            )

        # Plan execution
        logger.info(f"Planning expression: {expr_str}")
        steps = self.parser.plan(expr_str)

        # Execute steps sequentially
        current_lf = lf
        for var_name, sub_expr_str in steps:
            logger.info(f"Executing step: {var_name} = {sub_expr_str}")
            sub_expr = self.parser.parse(sub_expr_str)

            # If it's the final result, select it
            if var_name == "result":
                final_expr = sub_expr.alias("factor_value")
                return current_lf.select(
                    [pl.col("date"), pl.col("symbol"), final_expr]
                ).collect()
            else:
                # Intermediate step: compute and materialize
                # We use with_columns to add the intermediate result
                # IMPORTANT: We MUST collect() to break the Lazy graph and force
                # materialization to avoid the nested window function issue
                # (Polars #25691). Polars treats nested `over()` as hierarchical,
                # but TS/CS ops are orthogonal. Explicit materialization ensures
                # the inner result is fully computed before the outer window
                # function runs.

                temp_df = current_lf.with_columns(sub_expr.alias(var_name)).collect()
                current_lf = temp_df.lazy()

        # Should not reach here
        return pl.DataFrame()

    def run_batch(self, exprs: List[str]) -> pl.DataFrame:
        """Run multiple expressions at once."""
        lf = self.load_data()
        lf = self._ensure_date_column(lf)

        # We need a base dataframe with all dates/symbols to join against
        # This can be expensive if data is huge.
        # Alternatively, we can use the first result as base, but outer join might be
        # needed if filters differ (here filters are global).

        # Optimization: Scan unique date/symbol only
        base_df = (
            lf.select(["date", "symbol"]).unique().collect().sort(["date", "symbol"])
        )

        for i, expr_str in enumerate(exprs):
            # Run each expression.
            # Note: run_on_data sorts internally, so order is consistent.
            res = self.run_on_data(lf, expr_str)
            res = res.rename({"factor_value": f"factor_{i}"})

            # Join back to base
            # Use left join if base has all keys
            base_df = base_df.join(res, on=["date", "symbol"], how="left")

        return base_df



import importlib.util
from pathlib import Path

import pandas as pd
import pytest
from akquant import Bar, Strategy, run_backtest
from akquant.plot import (
    plot_dashboard,
    plot_pnl_vs_duration,
    plot_trades_distribution,
)
from akquant.plot.analysis import plot_returns_distribution, plot_rolling_metrics


class RoundTripStrategy(Strategy):
    """Generate one round-trip trade for report and plot checks."""

    def __init__(self) -> None:
        """Initialize internal step counter."""
        super().__init__()
        self.step = 0

    def on_bar(self, bar: Bar) -> None:
        """Buy once and sell once."""
        self.step += 1
        if self.step == 1:
            self.buy(symbol=bar.symbol, quantity=100, tag="entry")
        elif self.step == 3:
            self.sell(symbol=bar.symbol, quantity=100, tag="exit")


class NoTradeStrategy(Strategy):
    """Produce no trades for empty-trade report branch."""

    def on_bar(self, bar: Bar) -> None:
        """Do nothing on each bar."""
        _ = bar


def _build_data(symbol: str = "TEST", n: int = 5) -> list[Bar]:
    """Build deterministic daily bars."""
    data: list[Bar] = []
    for i in range(n):
        close = 10.0 + i
        data.append(
            Bar(
                timestamp=pd.Timestamp(f"2023-01-{i + 1:02d} 10:00:00").value,
                open=close,
                high=close + 0.2,
                low=close - 0.2,
                close=close,
                volume=10000.0,
                symbol=symbol,
            )
        )
    return data


def _skip_if_no_plotly() -> None:
    """Skip tests if plotly is unavailable."""
    if importlib.util.find_spec("plotly") is None:
        pytest.skip("plotly is required for report/plot tests")


def test_report_contains_new_analysis_sections(tmp_path: Path) -> None:
    """Report HTML should include attribution and capacity sections."""
    _skip_if_no_plotly()
    result = run_backtest(
        data=_build_data(),
        strategy=RoundTripStrategy,
        symbol="TEST",
        initial_cash=200000.0,
        commission_rate=0.0,
        stamp_tax_rate=0.0,
        transfer_fee_rate=0.0,
        min_commission=0.0,
        execution_mode="current_close",
        lot_size=1,
        show_progress=False,
    )

    report_path = tmp_path / "report_with_analysis.html"
    result.report(filename=str(report_path), show=False)
    html = report_path.read_text(encoding="utf-8")
    assert "组合归因与容量分析 (Attribution & Capacity)" in html
    assert "最新净暴露比 (Latest Net Exposure %)" in html
    assert "平均换手率 (Avg Turnover)" in html
    assert "暂无策略级风控拒单占比图" in html
    assert "暂无策略级拒单原因占比图" in html
    assert "暂无按日风控拒单趋势图" in html
    assert "暂无按策略风控拒单趋势图" in html
    assert "暂无按日拒单原因趋势图" in html


def test_report_handles_empty_trade_analysis_blocks(tmp_path: Path) -> None:
    """Report should still render when there are no trades."""
    _skip_if_no_plotly()
    result = run_backtest(
        data=_build_data(),
        strategy=NoTradeStrategy,
        symbol="TEST",
        initial_cash=200000.0,
        commission_rate=0.0,
        stamp_tax_rate=0.0,
        transfer_fee_rate=0.0,
        min_commission=0.0,
        execution_mode="current_close",
        lot_size=1,
        show_progress=False,
    )

    report_path = tmp_path / "report_empty_trades.html"
    result.report(filename=str(report_path), show=False)
    html = report_path.read_text(encoding="utf-8")
    assert "暂无归因数据" in html


def test_plot_functions_return_figures_for_non_empty_result() -> None:
    """Core plot functions should return figures for non-empty inputs."""
    _skip_if_no_plotly()
    result = run_backtest(
        data=_build_data(),
        strategy=RoundTripStrategy,
        symbol="TEST",
        initial_cash=200000.0,
        commission_rate=0.0,
        stamp_tax_rate=0.0,
        transfer_fee_rate=0.0,
        min_commission=0.0,
        execution_mode="current_close",
        lot_size=1,
        show_progress=False,
    )

    fig_dashboard = plot_dashboard(result, show=False)
    assert fig_dashboard is not None
    fig_trades = plot_trades_distribution(result.trades_df)
    assert fig_trades is not None
    fig_duration = plot_pnl_vs_duration(result.trades_df)
    assert fig_duration is not None
    fig_rolling = plot_rolling_metrics(result.daily_returns)
    assert fig_rolling is not None
    fig_returns = plot_returns_distribution(result.daily_returns)
    assert fig_returns is not None



from __future__ import annotations

import math
from dataclasses import dataclass, field

import akquant as aq
import pandas as pd
from akquant import Bar, Strategy


def build_data(n: int = 320) -> pd.DataFrame:
    """Build synthetic daily bars for live console rendering."""
    ts = pd.date_range("2022-01-01", periods=n, freq="D")
    close = [
        100.0 + 0.05 * i + 2.5 * math.sin(i / 8.2) + 1.3 * math.sin(i / 2.9)
        for i in range(n)
    ]
    return pd.DataFrame(
        {
            "date": ts,
            "open": [c * 0.998 for c in close],
            "high": [c * 1.004 for c in close],
            "low": [c * 0.996 for c in close],
            "close": close,
            "volume": [1200.0 + float(i % 30) * 18.0 for i in range(n)],
            "symbol": "LIVE_STREAM",
        }
    )


class LiveDemoStrategy(Strategy):
    """Simple MA crossover strategy for stream event demo."""

    def __init__(self) -> None:
        """Initialize warmup setting."""
        super().__init__()
        self.warmup_period = 20

    def on_bar(self, bar: Bar) -> None:
        """Trade by short/long moving average relation."""
        closes = self.get_history(20, bar.symbol, "close")
        if len(closes) < 20:
            return
        short = float(sum(closes[-5:])) / 5.0
        long = float(sum(closes)) / 20.0
        pos = self.get_position(bar.symbol)
        if short > long and pos == 0:
            self.order_target_percent(symbol=bar.symbol, target_percent=0.9)
        elif short < long and pos > 0:
            self.close_position(symbol=bar.symbol)


def sparkline(values: list[float], width: int = 42) -> str:
    """Convert a sequence of values to unicode sparkline."""
    if not values:
        return ""
    window = values[-width:]
    lo = min(window)
    hi = max(window)
    blocks = "▁▂▃▄▅▆▇█"
    if hi <= lo:
        return blocks[0] * len(window)
    span = hi - lo
    out: list[str] = []
    for v in window:
        idx = int((v - lo) / span * (len(blocks) - 1))
        out.append(blocks[idx])
    return "".join(out)


@dataclass
class LiveState:
    """Mutable stream state for console chart and alerts."""

    equities: list[float] = field(default_factory=list)
    peak: float = 0.0
    max_drawdown: float = 0.0
    progress_events: int = 0
    alert_emitted: bool = False


def run_live_console_demo() -> None:
    """Run backtest and render live chart plus warning messages."""
    state = LiveState()

    def on_event(event: aq.BacktestStreamEvent) -> None:
        event_type = str(event.get("event_type", "unknown"))
        payload = event.get("payload", {})
        if not isinstance(payload, dict):
            payload = {}

        if event_type == "started":
            print(f"[started] run_id={event.get('run_id')}")
            return

        if event_type == "progress":
            state.progress_events += 1
            if state.progress_events % 10 == 0:
                print(f"[progress] progress_events={state.progress_events}")
            return

        if event_type == "equity":
            raw = payload.get("equity")
            try:
                equity = float(str(raw))
            except Exception:
                return
            state.equities.append(equity)
            state.peak = max(state.peak, equity)
            if state.peak > 0:
                dd = (equity - state.peak) / state.peak
                state.max_drawdown = min(state.max_drawdown, dd)
                if dd <= -0.03 and not state.alert_emitted:
                    state.alert_emitted = True
                    print(f"\n[alert] drawdown={dd:.2%}")
            line = sparkline(state.equities, width=42)
            print(
                f"\r[live] equity={equity:,.2f} dd={state.max_drawdown:.2%} {line}",
                end="",
                flush=True,
            )
            return

        if event_type == "finished":
            print()
            print(
                f"[finished] status={payload.get('status')} "
                f"callback_error_count={payload.get('callback_error_count', '0')}"
            )

    result = aq.run_backtest(
        data=build_data(),
        strategy=LiveDemoStrategy,
        symbol="LIVE_STREAM",
        show_progress=False,
        initial_cash=500000.0,
        commission_rate=0.0,
        stamp_tax_rate=0.0,
        transfer_fee_rate=0.0,
        min_commission=0.0,
        execution_mode=aq.ExecutionMode.CurrentClose,
        lot_size=1,
        on_event=on_event,
        stream_progress_interval=4,
        stream_equity_interval=2,
        stream_batch_size=16,
        stream_max_buffer=128,
        stream_error_mode="continue",
    )
    print(f"total_return={result.metrics.total_return:.6f}")
    print(f"max_drawdown_live={state.max_drawdown:.2%}")
    print("done_streaming_live_console")


def main() -> None:
    """Entry point for live streaming console demo."""
    run_live_console_demo()


if __name__ == "__main__":
    main()



from typing import Any

import numpy as np
import pandas as pd
from akquant import Bar, Order, OrderStatus, Strategy, Trade, run_backtest


class TrailingOrderStrategy(Strategy):
    """Trailing Stop / StopLimit 示例策略."""

    def __init__(self, trail_offset: float = 1.5, **kwargs: Any) -> None:
        """初始化策略."""
        self.trail_offset = trail_offset
        self.entry_order_id = ""
        self.trailing_order_id = ""
        self.last_close: dict[str, float] = {}

    def on_bar(self, bar: Bar) -> None:
        """K线回调."""
        self.last_close[bar.symbol] = float(bar.close)
        if (
            self.get_position(bar.symbol) == 0
            and not self.entry_order_id
            and not self.trailing_order_id
        ):
            self.entry_order_id = self.buy(bar.symbol, 100, tag="trail-entry")
            print(
                f"[{bar.timestamp_str}] 提交进场单: {bar.symbol}, close={bar.close:.2f}"
            )

    def on_trade(self, trade: Trade) -> None:
        """成交回调."""
        print(
            f"[{trade.timestamp}] 成交: {trade.side} {trade.quantity} "
            f"@ {trade.price:.2f}, oid={trade.order_id}"
        )
        if trade.order_id == self.entry_order_id:
            self.entry_order_id = ""
            ref_price = self.last_close.get(trade.symbol, float(trade.price))
            self.trailing_order_id = self.place_trailing_stop(
                symbol=trade.symbol,
                quantity=float(trade.quantity),
                trail_offset=self.trail_offset,
                side="Sell",
                trail_reference_price=ref_price,
                tag="trail-stop",
            )
            print(
                f"[{trade.timestamp}] 提交 trailing stop: symbol={trade.symbol}, "
                f"offset={self.trail_offset:.2f}, ref={ref_price:.2f}"
            )
        elif trade.order_id == self.trailing_order_id:
            self.trailing_order_id = ""

    def on_order(self, order: Order) -> None:
        """订单状态回调."""
        if order.id in (
            self.entry_order_id,
            self.trailing_order_id,
        ) and order.status in (OrderStatus.Cancelled, OrderStatus.Rejected):
            if order.id == self.entry_order_id:
                self.entry_order_id = ""
            if order.id == self.trailing_order_id:
                self.trailing_order_id = ""
        if order.status == OrderStatus.Cancelled:
            print(f"[{order.id}] 订单已取消")
        elif order.status == OrderStatus.Rejected:
            print(f"[{order.id}] 订单被拒绝")


def run_example() -> None:
    """运行示例."""
    dates = pd.date_range("2024-01-01 09:30:00", periods=220, freq="1min")
    up = np.linspace(100, 112, 140)
    down = np.linspace(112, 96, 80)
    prices = np.concatenate([up, down]) + np.random.normal(0, 0.15, 220)
    df = pd.DataFrame(
        {
            "open": prices,
            "high": prices + 0.4,
            "low": prices - 0.4,
            "close": prices,
            "volume": 1000,
            "symbol": "TEST_STOCK",
        },
        index=dates,
    )
    print("开始 Trailing Order 策略回测...")
    run_backtest(
        data={"TEST_STOCK": df},
        strategy=TrailingOrderStrategy,
        initial_cash=100000.0,
    )


if __name__ == "__main__":
    run_example()



from .base import GatewayBundle, MarketGateway, TraderGateway
from .ctp_adapter import CTPMarketAdapter, CTPTraderAdapter
from .ctp_native import CTPMarketGateway, CTPTraderGateway
from .factory import create_gateway_bundle
from .mapper import BrokerEventMapper, create_default_mapper
from .miniqmt import MiniQMTMarketGateway, MiniQMTTraderGateway
from .models import (
    UnifiedAccount,
    UnifiedErrorType,
    UnifiedExecutionReport,
    UnifiedOrderRequest,
    UnifiedOrderSnapshot,
    UnifiedOrderStatus,
    UnifiedPosition,
    UnifiedTrade,
)
from .ptrade import PTradeMarketGateway, PTradeTraderGateway
from .registry import (
    get_broker_builder,
    list_registered_brokers,
    register_broker,
    unregister_broker,
)

__all__ = [
    "MarketGateway",
    "TraderGateway",
    "GatewayBundle",
    "UnifiedOrderStatus",
    "UnifiedErrorType",
    "UnifiedExecutionReport",
    "UnifiedOrderRequest",
    "UnifiedOrderSnapshot",
    "UnifiedTrade",
    "UnifiedAccount",
    "UnifiedPosition",
    "BrokerEventMapper",
    "create_default_mapper",
    "CTPMarketGateway",
    "CTPTraderGateway",
    "CTPMarketAdapter",
    "CTPTraderAdapter",
    "MiniQMTMarketGateway",
    "MiniQMTTraderGateway",
    "PTradeMarketGateway",
    "PTradeTraderGateway",
    "create_gateway_bundle",
    "register_broker",
    "unregister_broker",
    "get_broker_builder",
    "list_registered_brokers",
]



# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import datetime
import typing

import numpy
import numpy.typing

import akquant

class AssetType:
    Stock: typing.ClassVar["AssetType"]
    Futures: typing.ClassVar["AssetType"]
    Option: typing.ClassVar["AssetType"]
    Fund: typing.ClassVar["AssetType"]
    Crypto: typing.ClassVar["AssetType"]
    Forex: typing.ClassVar["AssetType"]
    Index: typing.ClassVar["AssetType"]
    Bond: typing.ClassVar["AssetType"]
    Unknown: typing.ClassVar["AssetType"]

class ExecutionMode:
    NextOpen: typing.ClassVar["ExecutionMode"]
    CurrentClose: typing.ClassVar["ExecutionMode"]
    NextAverage: typing.ClassVar["ExecutionMode"]

class OrderStatus:
    New: typing.ClassVar["OrderStatus"]
    Submitted: typing.ClassVar["OrderStatus"]
    Accepted: typing.ClassVar["OrderStatus"]
    Rejected: typing.ClassVar["OrderStatus"]
    PartiallyFilled: typing.ClassVar["OrderStatus"]
    Filled: typing.ClassVar["OrderStatus"]
    Cancelled: typing.ClassVar["OrderStatus"]
    Expired: typing.ClassVar["OrderStatus"]

class OrderSide:
    Buy: typing.ClassVar["OrderSide"]
    Sell: typing.ClassVar["OrderSide"]

class OrderType:
    Market: typing.ClassVar["OrderType"]
    Limit: typing.ClassVar["OrderType"]
    StopMarket: typing.ClassVar["OrderType"]
    StopLimit: typing.ClassVar["OrderType"]
    OCO: typing.ClassVar["OrderType"]
    Bracket: typing.ClassVar["OrderType"]
    StopTrail: typing.ClassVar["OrderType"]
    StopTrailLimit: typing.ClassVar["OrderType"]

class OrderRole:
    Standalone: typing.ClassVar["OrderRole"]
    Entry: typing.ClassVar["OrderRole"]
    StopLoss: typing.ClassVar["OrderRole"]
    TakeProfit: typing.ClassVar["OrderRole"]
    TrailStop: typing.ClassVar["OrderRole"]
    TrailStopLimit: typing.ClassVar["OrderRole"]

class TimeInForce:
    GTC: typing.ClassVar["TimeInForce"]
    IOC: typing.ClassVar["TimeInForce"]
    FOK: typing.ClassVar["TimeInForce"]
    GTD: typing.ClassVar["TimeInForce"]

class TradingSession:
    Normal: typing.ClassVar["TradingSession"]
    PreMarket: typing.ClassVar["TradingSession"]
    PostMarket: typing.ClassVar["TradingSession"]

class CorporateActionType:
    Split: typing.ClassVar["CorporateActionType"]
    Dividend: typing.ClassVar["CorporateActionType"]

class OptionType:
    Call: typing.ClassVar["OptionType"]
    Put: typing.ClassVar["OptionType"]

class SettlementType:
    Physical: typing.ClassVar["SettlementType"]
    Cash: typing.ClassVar["SettlementType"]

class ATR:
    r"""平均真实波幅 (ATR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ATR": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 ATR 值
        """
        ...

class AVGPRICE:
    r"""平均价格指标 (AVGPRICE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "AVGPRICE": ...
    def update(
        self, open: float, high: float, low: float, close: float
    ) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param open: 开盘价
        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 AVGPRICE 值
        """
        ...

class TRANGE:
    r"""真实波幅 (TRANGE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "TRANGE": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 TRANGE 值
        """
        ...

class MEDPRICE:
    r"""中位价指标 (MEDPRICE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "MEDPRICE": ...
    def update(self, high: float, low: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :return: 当前 MEDPRICE 值
        """
        ...

class TYPPRICE:
    r"""典型价格指标 (TYPPRICE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "TYPPRICE": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 TYPPRICE 值
        """
        ...

class WCLPRICE:
    r"""加权收盘价指标 (WCLPRICE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "WCLPRICE": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 WCLPRICE 值
        """
        ...

class MIDPRICE:
    r"""中点价格指标 (MIDPRICE)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MIDPRICE": ...
    def update(self, high: float, low: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :return: 当前 MIDPRICE 值
        """
        ...

class BacktestResult:
    r"""
    回测结果.

    :ivar equity_curve: 权益曲线 [(timestamp, equity)]
    :ivar cash_curve: 现金曲线 [(timestamp, cash)]
    :ivar metrics: 绩效指标对象
    :ivar trade_metrics: 交易统计对象
    :ivar trades: 平仓交易列表
    :ivar snapshots: 每日持仓快照 [(timestamp, [snapshot])]
    :ivar orders: 订单列表
    :ivar executions: 成交列表
    """

    equity_curve: list[tuple[int, float]]
    cash_curve: list[tuple[int, float]]
    metrics: PerformanceMetrics
    trade_metrics: TradePnL
    trades: list[ClosedTrade]
    snapshots: list[tuple[int, list[PositionSnapshot]]]
    orders: list[Order]
    executions: list[Trade]
    metrics_df: typing.Any
    orders_df: typing.Any
    def get_trades_ipc(self) -> bytes:
        r"""Get trades as Arrow IPC bytes (Zero-Copy-ish to Python)."""
        ...

    def get_positions_ipc(self) -> bytes:
        r"""Get positions history as Arrow IPC bytes."""
        ...

    def get_trades_dict(self) -> typing.Any:
        r"""Get trades as a dictionary of columns for fast DataFrame creation."""
        ...

    def get_positions_dict(self) -> typing.Any:
        r"""Get positions history as a dictionary of columns."""
        ...

    def get_orders_dict(self) -> typing.Any:
        r"""Get orders as a dictionary of columns for fast DataFrame creation."""
        ...

class Bar:
    r"""
    K 线数据结构.

    :ivar timestamp: Unix 时间戳 (纳秒)
    :ivar open: 开盘价
    :ivar high: 最高价
    :ivar low: 最低价
    :ivar close: 收盘价
    :ivar volume: 成交量
    :ivar symbol: 标的代码
    """

    timestamp: int
    symbol: str
    extra: dict[str, float]
    open: float
    high: float
    low: float
    close: float
    volume: float
    timestamp_str: str
    def __new__(
        cls,
        timestamp: int,
        open: float,
        high: float,
        low: float,
        close: float,
        volume: float,
        symbol: str,
        extra: typing.Optional[dict[str, float]] = ...,
    ) -> "Bar": ...
    def set_open(self, value: float) -> None: ...
    def set_high(self, value: float) -> None: ...
    def set_low(self, value: float) -> None: ...
    def set_close(self, value: float) -> None: ...
    def set_volume(self, value: float) -> None: ...
    def __repr__(self) -> str: ...

class BarAggregator:
    r"""
    K 线合成器.

    用于将 Tick 数据合成为 K 线 (Bar) 数据。
    """

    def __new__(cls, feed: "DataFeed", interval_min: int = ...) -> "BarAggregator": ...
    def on_tick(
        self, symbol: str, price: float, volume: float, timestamp_ns: int
    ) -> None:
        r"""
        处理新的 Tick 数据.

        :param symbol: 标的代码
        :param price: 最新价
        :param volume: 累计成交量 (TotalVolume)
        :param timestamp_ns: 时间戳 (纳秒)
        """
        ...

class BollingerBands:
    r"""布林带 (Bollinger Bands)."""

    value: typing.Optional[tuple[float, float, float]]
    def __new__(cls, period: int, multiplier: float) -> "BollingerBands": ...
    def update(self, value: float) -> typing.Optional[tuple[float, float, float]]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: (上轨, 中轨, 下轨)
        """
        ...

class ClosedTrade:
    r"""
    平仓交易记录.

    :ivar symbol: 标的代码
    :ivar entry_time: 开仓时间戳 (纳秒)
    :ivar exit_time: 平仓时间戳 (纳秒)
    :ivar entry_price: 开仓价格
    :ivar exit_price: 平仓价格
    :ivar quantity: 交易数量
    :ivar side: 方向 ("long" 或 "short")
    :ivar pnl: 毛利 (扣除佣金前)
    :ivar net_pnl: 净利 (扣除佣金后)
    :ivar return_pct: 收益率 (如 0.05 表示 5%)
    :ivar commission: 佣金
    :ivar duration_bars: 持仓 K 线数
    :ivar duration: 持仓时长 (纳秒)
    :ivar mae: 最大不利变动 (MAE)
    :ivar mfe: 最大有利变动 (MFE)
    :ivar entry_tag: 开仓订单标签
    :ivar exit_tag: 平仓订单标签
    :ivar entry_portfolio_value: 开仓时账户权益
    :ivar max_drawdown_pct: 交易期间最大回撤 (%)
    """

    symbol: str
    entry_time: int
    exit_time: int
    entry_price: float
    exit_price: float
    quantity: float
    side: str
    pnl: float
    net_pnl: float
    return_pct: float
    commission: float
    duration_bars: int
    duration: int
    mae: float
    mfe: float
    entry_tag: str
    exit_tag: str
    entry_portfolio_value: float
    max_drawdown_pct: float
    def __repr__(self) -> str: ...

class CorporateAction:
    symbol: str
    date: datetime.date
    action_type: akquant.CorporateActionType
    value: float
    def __new__(
        cls,
        symbol: str,
        date: datetime.date,
        action_type: akquant.CorporateActionType,
        value: float,
    ) -> "CorporateAction": ...
    def set_value(self, value: float) -> None: ...

class DataFeed:
    r"""
    数据源管理器.

    负责管理历史数据和实时数据流。
    支持 CSV 文件读取、数组批量加载和实时数据推送。
    """

    def sort(self) -> None: ...
    def add_bars(self, bars: typing.Sequence[Bar]) -> None: ...
    def add_arrays(
        self,
        timestamps: numpy.ndarray,
        opens: numpy.ndarray,
        highs: numpy.ndarray,
        lows: numpy.ndarray,
        closes: numpy.ndarray,
        volumes: numpy.ndarray,
        symbol: str,
    ) -> None: ...
    def __new__(
        cls,
        start_date: typing.Optional[str] = ...,
        end_date: typing.Optional[str] = ...,
    ) -> "DataFeed": ...

class EMA:
    r"""指数移动平均线 (EMA)."""

    value: typing.Optional[float]
    is_ready: bool
    period: int
    def __new__(cls, period: int) -> "EMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 EMA 值
        """
        ...

class Engine:
    r"""
    主回测引擎.

    :ivar feed: 数据源
    :ivar portfolio: 投资组合
    :ivar orders: 订单列表
    :ivar trades: 成交列表
    """

    risk_manager: RiskManager
    orders: list[Order]
    trades: list[Trade]
    portfolio: Portfolio
    feed: DataFeed
    snapshots: list[tuple[int, list[PositionSnapshot]]]
    def __new__(cls) -> "Engine": ...
    def set_risk_config(self, config: RiskConfig) -> None:
        r"""
        设置风控配置.

        由于 PyO3 对嵌套结构体的属性访问可能返回副本，
        提供此方法以显式更新风控配置。

        :param config: 新的风控配置
        """
        ...

    def set_history_depth(self, depth: int) -> None:
        r"""
        设置历史数据长度.

        :param depth: 历史数据长度
        """
        ...

    def set_timezone(self, offset: int) -> None:
        r"""
        设置时区偏移 (秒).

        :param offset: 偏移秒数 (例如 UTC+8 为 28800)
        """
        ...

    def use_simulated_execution(self) -> None:
        r"""
        启用模拟执行 (回测模式).

        默认模式。在内存中撮合订单。
        """
        ...

    def use_realtime_execution(self) -> None:
        r"""
        启用实盘执行 (CTP/Broker 模式).

        模拟对接 CTP 或其他 Broker API。
        在此模式下，订单会被标记为 Submitted 并等待回调 (目前仅模拟发送)。
        """
        ...

    def register_custom_matcher(
        self, asset_type: akquant.AssetType, matcher: typing.Any
    ) -> None:
        r"""
        注册自定义撮合器 (Python).

        :param asset_type: 资产类型
        :param matcher: Python 撮合器对象 (需实现 match 方法)
        """
        ...

    def set_execution_mode(self, mode: akquant.ExecutionMode) -> None:
        r"""
        设置撮合模式.

        :param mode: 撮合模式 (ExecutionMode.CurrentClose 或 ExecutionMode.NextOpen)
        :type mode: ExecutionMode
        """
        ...

    def use_simple_market(self, commission_rate: float) -> None:
        r"""
        启用 SimpleMarket (7x24小时, T+0, 无税, 简单佣金).

        :param commission_rate: 佣金率
        """
        ...

    def use_china_market(self) -> None:
        r"""启用 ChinaMarket (支持 T+1/T+0, 印花税, 过户费, 交易时段等)."""
        ...

    def set_t_plus_one(self, enabled: bool) -> None:
        r"""
        启用/禁用 T+1 交易规则 (仅针对 ChinaMarket).

        :param enabled: 是否启用 T+1
        :type enabled: bool
        """
        ...

    def set_force_session_continuous(self, enabled: bool) -> None:
        r"""
        强制连续交易时段.

        :param enabled: 是否强制连续交易 (忽略午休等)
        """
        ...

    def add_corporate_action(self, action: CorporateAction) -> None:
        r"""
        添加公司行为 (除权除息).

        :param action: 公司行为对象
        """
        ...

    def use_china_futures_market(self) -> None:
        r"""
        启用中国期货市场默认配置.

        - 切换到 ChinaMarket
        - 设置 T+0
        - 保持当前交易时段配置 (需手动设置 set_market_sessions 以匹配特定品种)
        """
        ...

    def process_option_expiry(self, _local_date: datetime.date) -> None: ...
    def set_stock_fee_rules(
        self,
        commission_rate: float,
        stamp_tax: float,
        transfer_fee: float,
        min_commission: float,
    ) -> None:
        r"""
        设置股票费率规则 (按交易金额比例).

        :param commission_rate: 佣金率 (如 0.0003)
        :param stamp_tax: 印花税率 (如 0.001)
        :param transfer_fee: 过户费率 (如 0.00002)
        :param min_commission: 最低佣金 (如 5.0)
        """
        ...

    def set_future_fee_rules(self, commission_rate: float) -> None:
        r"""
        设置期货费率规则.

        :param commission_rate: 佣金率 (如 0.0001)
        """
        ...

    def set_fund_fee_rules(
        self, commission_rate: float, transfer_fee: float, min_commission: float
    ) -> None:
        r"""
        设置基金费率规则.

        :param commission_rate: 佣金率
        :param transfer_fee: 过户费率
        :param min_commission: 最低佣金
        """
        ...

    def set_option_fee_rules(self, commission_per_contract: float) -> None:
        r"""
        设置期权费率规则.

        :param commission_per_contract: 每张合约佣金 (如 5.0)
        """
        ...

    def set_crypto_fee_rules(self, commission_rate: float) -> None:
        r"""
        设置加密货币费率规则 (按金额比例).

        :param commission_rate: 佣金率 (如 0.001)
        """
        ...

    def set_forex_fee_rules(self, commission_rate: float) -> None:
        r"""
        设置外汇费率规则 (按金额比例).

        :param commission_rate: 佣金率 (如 0.00005)
        """
        ...

    def set_slippage(self, type_: str, value: float) -> None:
        r"""
        设置滑点模型.

        :param type: 滑点类型 ("fixed" 或 "percent")
        :param value: 滑点值 (固定金额 或 百分比如 0.001)
        """
        ...

    def set_volume_limit(self, limit: float) -> None:
        r"""
        设置成交量限制.

        :param limit: 限制比例 (0.0-1.0), 0.0 为不限制
        """
        ...

    def set_market_sessions(
        self, sessions: typing.Sequence[tuple[str, str, akquant.TradingSession]]
    ) -> None:
        r"""
        设置市场交易时段.

        :param sessions: 交易时段列表，每个元素为 (开始时间, 结束时间, 时段类型)
        :type sessions: List[Tuple[str, str, TradingSession]]

        示例::

        ```python
        engine.set_market_sessions([
            ("09:30:00", "11:30:00", TradingSession.Normal),
            ("13:00:00", "15:00:00", TradingSession.Normal)
        ])
        ```
        """
        ...

    def add_instrument(self, instrument: Instrument) -> None:
        r"""
        添加交易标的.

        :param instrument: 交易标的对象
        :type instrument: Instrument
        """
        ...

    def set_cash(self, cash: float) -> None:
        r"""
        设置初始资金.

        :param cash: 初始资金数额
        :type cash: float
        """
        ...

    def add_data(self, feed: DataFeed) -> None:
        r"""
        添加数据源.

        :param feed: 数据源对象
        :type feed: DataFeed
        """
        ...

    def add_bars(self, bars: typing.Sequence[Bar]) -> None:
        r"""
        批量添加 K 线数据.

        :param bars: K 线列表
        """
        ...

    def set_stream_callback(self, callback: typing.Any) -> None: ...
    def clear_stream_callback(self) -> None: ...
    def set_stream_options(
        self,
        progress_interval: int,
        equity_interval: int,
        batch_size: int,
        max_buffer: int,
        error_mode: str,
        stream_mode: str,
    ) -> None: ...
    def set_default_strategy_id(self, strategy_id: typing.Optional[str]) -> None: ...
    def get_default_strategy_id(self) -> typing.Optional[str]: ...
    def get_strategy_slot_ids(self) -> list[str]: ...
    def get_strategy_slots(self) -> list[str]: ...
    def get_active_strategy_slot(self) -> int: ...
    def set_strategy_slots(self, slot_ids: list[str]) -> None: ...
    def set_strategy_for_slot(self, slot_index: int, strategy: typing.Any) -> None: ...
    def clear_strategy_for_slot(self, slot_index: int) -> None: ...
    def set_strategy_max_order_value_limits(self, limits: dict[str, float]) -> None: ...
    def set_strategy_max_order_size_limits(self, limits: dict[str, float]) -> None: ...
    def set_strategy_max_position_size_limits(
        self, limits: dict[str, float]
    ) -> None: ...
    def set_strategy_max_daily_loss_limits(self, limits: dict[str, float]) -> None: ...
    def set_strategy_max_drawdown_limits(self, limits: dict[str, float]) -> None: ...
    def set_strategy_reduce_only_after_risk(self, flags: dict[str, bool]) -> None: ...
    def set_strategy_risk_cooldown_bars(self, bars: dict[str, int]) -> None: ...
    def set_strategy_priorities(self, priorities: dict[str, int]) -> None: ...
    def set_strategy_risk_budget_limits(self, limits: dict[str, float]) -> None: ...
    def set_portfolio_risk_budget_limit(
        self, limit: typing.Optional[float]
    ) -> None: ...
    def set_risk_budget_mode(self, mode: str) -> None: ...
    def set_risk_budget_reset_daily(self, enabled: bool) -> None: ...
    def run(self, strategy: typing.Any, show_progress: bool) -> str:
        r"""
        运行回测.

        :param strategy: 策略对象
        :param show_progress: 是否显示进度条
        :type strategy: object
        :type show_progress: bool
        :return: 回测结果摘要
        :rtype: str
        """
        ...

    def get_results(self) -> BacktestResult:
        r"""
        获取回测结果.

        :return: BacktestResult
        """
        ...

class Instrument:
    r"""
    交易标的.

    :ivar symbol: 代码
    :ivar asset_type: 资产类型
    :ivar multiplier: 合约乘数
    :ivar margin_ratio: 保证金比率
    :ivar tick_size: 最小变动价位
    """

    asset_type: akquant.AssetType
    symbol: str
    multiplier: float
    margin_ratio: float
    lot_size: float
    tick_size: float
    def __new__(
        cls,
        symbol: str,
        asset_type: AssetType,
        multiplier: float = ...,
        margin_ratio: float = ...,
        tick_size: float = ...,
        option_type: typing.Optional[OptionType] = ...,
        strike_price: typing.Optional[float] = ...,
        expiry_date: typing.Optional[int] = ...,
        lot_size: float = ...,
        underlying_symbol: typing.Optional[str] = ...,
        settlement_type: typing.Optional[SettlementType] = ...,
    ) -> "Instrument": ...

class MACD:
    r"""平滑异同移动平均线 (MACD)."""

    value: typing.Optional[tuple[float, float, float]]
    def __new__(
        cls, fast_period: int, slow_period: int, signal_period: int
    ) -> "MACD": ...
    def update(self, value: float) -> typing.Optional[tuple[float, float, float]]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: (DIF, DEA, MACD柱)
        """
        ...

class APO:
    r"""绝对价格震荡指标 (APO)."""

    value: typing.Optional[float]
    def __new__(cls, fast_period: int, slow_period: int) -> "APO": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 APO 值
        """
        ...

class PPO:
    r"""百分比价格震荡指标 (PPO)."""

    value: typing.Optional[float]
    def __new__(cls, fast_period: int, slow_period: int) -> "PPO": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 PPO 值
        """
        ...

class Order:
    r"""
    订单.

    :ivar id: 订单ID
    :ivar symbol: 标的代码
    :ivar side: 交易方向
    :ivar order_type: 订单类型
    :ivar quantity: 数量
    :ivar price: 价格 (限价单有效)
    :ivar time_in_force: 订单有效期
    :ivar trigger_price: 触发价格 (止损/止盈单)
    :ivar status: 订单状态
    :ivar filled_quantity: 已成交数量
    :ivar average_filled_price: 成交均价
    """

    id: str
    symbol: str
    side: akquant.OrderSide
    order_type: akquant.OrderType
    time_in_force: akquant.TimeInForce
    status: akquant.OrderStatus
    created_at: int
    updated_at: int
    tag: str
    reject_reason: str
    owner_strategy_id: typing.Optional[str]
    graph_id: typing.Optional[str]
    parent_order_id: typing.Optional[str]
    order_role: akquant.OrderRole
    trail_offset: typing.Optional[float]
    trail_reference_price: typing.Optional[float]
    commission: float
    quantity: float
    price: typing.Optional[float]
    trigger_price: typing.Optional[float]
    filled_quantity: float
    average_filled_price: typing.Optional[float]
    def __new__(
        cls,
        id: str,
        symbol: str,
        side: akquant.OrderSide,
        order_type: akquant.OrderType,
        quantity: float,
        price: typing.Optional[float] = ...,
        time_in_force: typing.Optional[akquant.TimeInForce] = ...,
        trigger_price: typing.Optional[float] = ...,
        created_at: int = ...,
        tag: str = ...,
        owner_strategy_id: typing.Optional[str] = ...,
        graph_id: typing.Optional[str] = ...,
        parent_order_id: typing.Optional[str] = ...,
        order_role: typing.Optional[akquant.OrderRole] = ...,
        trail_offset: typing.Optional[float] = ...,
        trail_reference_price: typing.Optional[float] = ...,
    ) -> "Order": ...
    def set_trail_offset(self, value: typing.Optional[float]) -> None: ...
    def set_trail_reference_price(self, value: typing.Optional[float]) -> None: ...
    def set_filled_quantity(self, value: float) -> None: ...
    def set_average_filled_price(self, value: typing.Optional[float]) -> None: ...
    def __repr__(self) -> str: ...

class PerformanceMetrics:
    r"""
    绩效指标.

    :ivar total_return: 总收益 (数值)
    :ivar annualized_return: 年化收益率
    :ivar max_drawdown: 最大回撤 (百分比, 如 0.15 表示 15%)
    :ivar max_drawdown_value: 最大回撤金额
    :ivar max_drawdown_pct: 最大回撤百分比 (如 15.0 表示 15%)
    :ivar sharpe_ratio: 夏普比率
    :ivar sortino_ratio: 索提诺比率
    :ivar calmar_ratio: 卡玛比率
    :ivar volatility: 年化波动率
    :ivar ulcer_index: 溃疡指数
    :ivar upi: 溃疡绩效指数 (UPI)
    :ivar equity_r2: 权益曲线 R^2 (拟合度)
    :ivar std_error: 标准误差
    :ivar win_rate: 胜率 (0.0-1.0)
    :ivar initial_market_value: 初始市值
    :ivar end_market_value: 结束市值
    :ivar total_return_pct: 总收益率 (%)
    :ivar start_time: 开始时间戳 (纳秒)
    :ivar end_time: 结束时间戳 (纳秒)
    :ivar duration: 持续时间 (纳秒)
    :ivar total_bars: 总 K 线数
    :ivar exposure_time_pct: 市场暴露时间百分比
    :ivar var_95: 95% VaR (在风险价值)
    :ivar var_99: 99% VaR
    :ivar cvar_95: 95% CVaR (条件在风险价值)
    :ivar cvar_99: 99% CVaR
    """

    total_return: float
    annualized_return: float
    max_drawdown: float
    max_drawdown_value: float
    max_drawdown_pct: float
    sharpe_ratio: float
    sortino_ratio: float
    calmar_ratio: float
    volatility: float
    ulcer_index: float
    upi: float
    equity_r2: float
    std_error: float
    win_rate: float
    initial_market_value: float
    end_market_value: float
    total_return_pct: float
    start_time: int
    end_time: int
    duration: int
    total_bars: int
    exposure_time_pct: float
    var_95: float
    var_99: float
    cvar_95: float
    cvar_99: float
    def __repr__(self) -> str: ...
    def __getitem__(self, key: str) -> typing.Any: ...

class Portfolio:
    r"""
    投资组合管理.

    :ivar cash: 当前现金余额
    :ivar positions: 当前持仓 (symbol -> quantity)
    :ivar available_positions: 可用持仓 (symbol -> quantity)
    :ivar total_equity: 总权益 (动态计算)
    """

    cash: float
    positions: dict[str, float]
    available_positions: dict[str, float]

    def __new__(cls, cash: float) -> "Portfolio": ...
    def get_cash(self) -> float: ...
    def get_positions(self) -> dict[str, float]: ...
    def get_available_positions(self) -> dict[str, float]: ...
    def get_position(self, symbol: str) -> float: ...
    def get_available_position(self, symbol: str) -> float: ...
    def __repr__(self) -> str: ...

class PositionSnapshot:
    r"""
    每日持仓快照.

    :ivar symbol: 标的代码
    :ivar quantity: 持仓数量
    :ivar entry_price: 开仓均价
    :ivar long_shares: 多头持仓数量
    :ivar short_shares: 空头持仓数量
    :ivar close: 收盘价
    :ivar equity: 账户权益
    :ivar market_value: 市值
    :ivar margin: 保证金
    :ivar unrealized_pnl: 未实现盈亏
    """

    symbol: str
    quantity: float
    entry_price: float
    long_shares: float
    short_shares: float
    close: float
    equity: float
    market_value: float
    margin: float
    unrealized_pnl: float
    def __repr__(self) -> str: ...

class RSI:
    r"""相对强弱指数 (RSI)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "RSI": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点 (通常是收盘价)
        :return: 当前 RSI 值
        """
        ...

class ROC:
    r"""变化率指标 (ROC)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ROC": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ROC 值
        """
        ...

class ROCP:
    r"""变化率百分比指标 (ROCP)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ROCP": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ROCP 值
        """
        ...

class ROCR:
    r"""变化率比值指标 (ROCR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ROCR": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ROCR 值
        """
        ...

class ROCR100:
    r"""变化率比值指标 (ROCR100)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ROCR100": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ROCR100 值
        """
        ...

class MOM:
    r"""动量指标 (MOM)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MOM": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 MOM 值
        """
        ...

class CMO:
    r"""钱德动量摆动指标 (CMO)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "CMO": ...
    def update(self, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param close: 收盘价
        :return: 当前 CMO 值
        """
        ...

class OBV:
    r"""能量潮指标 (OBV)."""

    value: typing.Optional[float]
    def __new__(cls) -> "OBV": ...
    def update(self, close: float, volume: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param close: 收盘价
        :param volume: 成交量
        :return: 当前 OBV 值
        """
        ...

class AD:
    r"""蔡金累积/派发线 (AD)."""

    value: typing.Optional[float]
    def __new__(cls) -> "AD": ...
    def update(
        self, high: float, low: float, close: float, volume: float
    ) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :param volume: 成交量
        :return: 当前 AD 值
        """
        ...

class ADOSC:
    r"""蔡金累积/派发振荡器 (ADOSC)."""

    value: typing.Optional[float]
    def __new__(cls, fast_period: int, slow_period: int) -> "ADOSC": ...
    def update(
        self, high: float, low: float, close: float, volume: float
    ) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :param volume: 成交量
        :return: 当前 ADOSC 值
        """
        ...

class BOP:
    r"""均势指标 (BOP)."""

    value: typing.Optional[float]
    def __new__(cls) -> "BOP": ...
    def update(
        self, open: float, high: float, low: float, close: float
    ) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param open: 开盘价
        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 BOP 值
        """
        ...

class DEMA:
    r"""双指数移动平均线 (DEMA)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "DEMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 DEMA 值
        """
        ...

class TRIX:
    r"""三重指数平滑变动率 (TRIX)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "TRIX": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 TRIX 值
        """
        ...

class MFI:
    r"""资金流量指标 (MFI)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MFI": ...
    def update(
        self, high: float, low: float, close: float, volume: float
    ) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :param volume: 成交量
        :return: 当前 MFI 值
        """
        ...

class TEMA:
    r"""三重指数移动平均线 (TEMA)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "TEMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 TEMA 值
        """
        ...

class KAMA:
    r"""考夫曼自适应均线 (KAMA)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "KAMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 KAMA 值
        """
        ...

class NATR:
    r"""标准化真实波幅均值 (NATR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "NATR": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 NATR 值
        """
        ...

class STDDEV:
    r"""标准差指标 (STDDEV)."""

    value: typing.Optional[float]
    def __new__(cls, period: int, nbdev: float) -> "STDDEV": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 STDDEV 值
        """
        ...

class VAR:
    r"""方差指标 (VAR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int, nbdev: float) -> "VAR": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 VAR 值
        """
        ...

class SAR:
    r"""抛物线转向指标 (SAR)."""

    value: typing.Optional[float]
    def __new__(cls, acceleration: float, maximum: float) -> "SAR": ...
    def update(self, high: float, low: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :return: 当前 SAR 值
        """
        ...

class CCI:
    r"""顺势指标 (CCI)."""

    value: typing.Optional[float]
    def __new__(cls, period: int, c: float) -> "CCI": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 CCI 值
        """
        ...

class ADX:
    r"""平均趋向指标 (ADX)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ADX": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 ADX 值
        """
        ...

class DX:
    r"""动向指数 (DX)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "DX": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 DX 值
        """
        ...

class ADXR:
    r"""平均趋向指标评分 (ADXR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "ADXR": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 ADXR 值
        """
        ...

class PLUS_DI:
    r"""正向动向指标 (+DI)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "PLUS_DI": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 +DI 值
        """
        ...

class MINUS_DI:
    r"""负向动向指标 (-DI)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MINUS_DI": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 -DI 值
        """
        ...

class ULTOSC:
    r"""终极振荡器 (ULTOSC)."""

    value: typing.Optional[float]
    def __new__(cls, period1: int, period2: int, period3: int) -> "ULTOSC": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 ULTOSC 值
        """
        ...

class AROON:
    r"""阿隆指标 (AROON)."""

    value: typing.Optional[tuple[float, float]]
    def __new__(cls, period: int) -> "AROON": ...
    def update(self, high: float, low: float) -> typing.Optional[tuple[float, float]]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :return: (AroonDown, AroonUp)
        """
        ...

class AROONOSC:
    r"""阿隆振荡器 (AROONOSC)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "AROONOSC": ...
    def update(self, high: float, low: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :return: 当前 AROONOSC 值
        """
        ...

class LINEARREG:
    r"""线性回归端点值 (LINEARREG)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "LINEARREG": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LINEARREG 值
        """
        ...

class LINEARREG_SLOPE:
    r"""线性回归斜率 (LINEARREG_SLOPE)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "LINEARREG_SLOPE": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LINEARREG_SLOPE 值
        """
        ...

class LINEARREG_R2:
    r"""线性回归决定系数 (LINEARREG_R2)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "LINEARREG_R2": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LINEARREG_R2 值
        """
        ...

class HT_TRENDLINE:
    r"""希尔伯特趋势线 (HT_TRENDLINE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "HT_TRENDLINE": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 HT_TRENDLINE 值
        """
        ...

class LINEARREG_INTERCEPT:
    r"""线性回归截距 (LINEARREG_INTERCEPT)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "LINEARREG_INTERCEPT": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LINEARREG_INTERCEPT 值
        """
        ...

class LINEARREG_ANGLE:
    r"""线性回归角度 (LINEARREG_ANGLE)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "LINEARREG_ANGLE": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LINEARREG_ANGLE 值
        """
        ...

class TSF:
    r"""时间序列预测 (TSF)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "TSF": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 TSF 值
        """
        ...

class CORREL:
    r"""相关系数指标 (CORREL)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "CORREL": ...
    def update(self, x: float, y: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param x: 第一个序列值
        :param y: 第二个序列值
        :return: 当前 CORREL 值
        """
        ...

class BETA:
    r"""贝塔系数指标 (BETA)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "BETA": ...
    def update(self, x: float, y: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param x: 第一个序列值
        :param y: 第二个序列值
        :return: 当前 BETA 值
        """
        ...

class COVAR:
    r"""协方差指标 (COVAR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "COVAR": ...
    def update(self, x: float, y: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param x: 第一个序列值
        :param y: 第二个序列值
        :return: 当前 COVAR 值
        """
        ...

class STOCH:
    r"""随机指标 (STOCH)."""

    value: typing.Optional[tuple[float, float]]
    def __new__(
        cls, fastk_period: int, slowk_period: int, slowd_period: int
    ) -> "STOCH": ...
    def update(
        self, high: float, low: float, close: float
    ) -> typing.Optional[tuple[float, float]]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 (slowk, slowd)
        """
        ...

class RiskConfig:
    r"""风控配置."""

    restricted_list: list[str]
    max_order_size: typing.Optional[float]
    max_order_value: typing.Optional[float]
    max_position_size: typing.Optional[float]
    max_position_pct: typing.Optional[float]
    active: bool
    check_cash: bool
    safety_margin: float
    def __new__(cls) -> "RiskConfig": ...

class RiskManager:
    config: RiskConfig
    def add_max_position_percent_rule(self, pct: float) -> None: ...
    def add_sector_concentration_rule(
        self, limit: float, sector_map: typing.Mapping[str, str]
    ) -> None: ...
    def add_max_drawdown_rule(self, limit: float) -> None: ...
    def add_max_daily_loss_rule(self, limit: float) -> None: ...
    def add_stop_loss_rule(self, threshold: float) -> None: ...

class SMA:
    r"""简单移动平均线 (SMA)."""

    value: typing.Optional[float]
    is_ready: bool
    def __new__(cls, period: int) -> "SMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 SMA 值 (如果数据不足则返回 None)
        """
        ...

class WMA:
    r"""加权移动平均线 (WMA)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "WMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 WMA 值
        """
        ...

class TRIMA:
    r"""三角移动平均线 (TRIMA)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "TRIMA": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 TRIMA 值
        """
        ...

class MIDPOINT:
    r"""中点指标 (MIDPOINT)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MIDPOINT": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 MIDPOINT 值
        """
        ...

class MAX:
    r"""滚动最大值指标 (MAX)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MAX": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 MAX 值
        """
        ...

class MIN:
    r"""滚动最小值指标 (MIN)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MIN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 MIN 值
        """
        ...

class MAXINDEX:
    r"""滚动最大值索引指标 (MAXINDEX)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MAXINDEX": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 MAXINDEX 值
        """
        ...

class MININDEX:
    r"""滚动最小值索引指标 (MININDEX)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "MININDEX": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 MININDEX 值
        """
        ...

class MINMAXINDEX:
    r"""滚动最值索引双输出指标 (MINMAXINDEX)."""

    value: typing.Optional[tuple[float, float]]
    def __new__(cls, period: int) -> "MINMAXINDEX": ...
    def update(self, value: float) -> typing.Optional[tuple[float, float]]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 (MININDEX, MAXINDEX)
        """
        ...

class MINMAX:
    r"""滚动最值双输出指标 (MINMAX)."""

    value: typing.Optional[tuple[float, float]]
    def __new__(cls, period: int) -> "MINMAX": ...
    def update(self, value: float) -> typing.Optional[tuple[float, float]]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 (MIN, MAX)
        """
        ...

class SUM:
    r"""滚动求和指标 (SUM)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "SUM": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 SUM 值
        """
        ...

class AVGDEV:
    r"""平均绝对偏差指标 (AVGDEV)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "AVGDEV": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 AVGDEV 值
        """
        ...

class RANGE:
    r"""滚动高低差指标 (RANGE)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "RANGE": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 RANGE 值
        """
        ...

class LN:
    r"""自然对数变换指标 (LN)."""

    value: typing.Optional[float]
    def __new__(cls) -> "LN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LN 值
        """
        ...

class LOG10:
    r"""常用对数变换指标 (LOG10)."""

    value: typing.Optional[float]
    def __new__(cls) -> "LOG10": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 LOG10 值
        """
        ...

class SQRT:
    r"""平方根变换指标 (SQRT)."""

    value: typing.Optional[float]
    def __new__(cls) -> "SQRT": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 SQRT 值
        """
        ...

class CEIL:
    r"""向上取整变换指标 (CEIL)."""

    value: typing.Optional[float]
    def __new__(cls) -> "CEIL": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 CEIL 值
        """
        ...

class FLOOR:
    r"""向下取整变换指标 (FLOOR)."""

    value: typing.Optional[float]
    def __new__(cls) -> "FLOOR": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 FLOOR 值
        """
        ...

class SIN:
    r"""正弦变换指标 (SIN)."""

    value: typing.Optional[float]
    def __new__(cls) -> "SIN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 SIN 值
        """
        ...

class COS:
    r"""余弦变换指标 (COS)."""

    value: typing.Optional[float]
    def __new__(cls) -> "COS": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 COS 值
        """
        ...

class TAN:
    r"""正切变换指标 (TAN)."""

    value: typing.Optional[float]
    def __new__(cls) -> "TAN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 TAN 值
        """
        ...

class ASIN:
    r"""反正弦变换指标 (ASIN)."""

    value: typing.Optional[float]
    def __new__(cls) -> "ASIN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ASIN 值
        """
        ...

class ACOS:
    r"""反余弦变换指标 (ACOS)."""

    value: typing.Optional[float]
    def __new__(cls) -> "ACOS": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ACOS 值
        """
        ...

class ATAN:
    r"""反正切变换指标 (ATAN)."""

    value: typing.Optional[float]
    def __new__(cls) -> "ATAN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ATAN 值
        """
        ...

class SINH:
    r"""双曲正弦变换指标 (SINH)."""

    value: typing.Optional[float]
    def __new__(cls) -> "SINH": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 SINH 值
        """
        ...

class COSH:
    r"""双曲余弦变换指标 (COSH)."""

    value: typing.Optional[float]
    def __new__(cls) -> "COSH": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 COSH 值
        """
        ...

class TANH:
    r"""双曲正切变换指标 (TANH)."""

    value: typing.Optional[float]
    def __new__(cls) -> "TANH": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 TANH 值
        """
        ...

class EXP:
    r"""指数变换指标 (EXP)."""

    value: typing.Optional[float]
    def __new__(cls) -> "EXP": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 EXP 值
        """
        ...

class ABS:
    r"""绝对值变换指标 (ABS)."""

    value: typing.Optional[float]
    def __new__(cls) -> "ABS": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 ABS 值
        """
        ...

class SIGN:
    r"""符号变换指标 (SIGN)."""

    value: typing.Optional[float]
    def __new__(cls) -> "SIGN": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 SIGN 值
        """
        ...

class ADD:
    r"""逐元素加法指标 (ADD)."""

    value: typing.Optional[float]
    def __new__(cls) -> "ADD": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 ADD 值
        """
        ...

class SUB:
    r"""逐元素减法指标 (SUB)."""

    value: typing.Optional[float]
    def __new__(cls) -> "SUB": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 SUB 值
        """
        ...

class MULT:
    r"""逐元素乘法指标 (MULT)."""

    value: typing.Optional[float]
    def __new__(cls) -> "MULT": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 MULT 值
        """
        ...

class DIV:
    r"""逐元素除法指标 (DIV)."""

    value: typing.Optional[float]
    def __new__(cls) -> "DIV": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 DIV 值
        """
        ...

class MAX2:
    r"""逐元素最大值指标 (MAX2)."""

    value: typing.Optional[float]
    def __new__(cls) -> "MAX2": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 MAX2 值
        """
        ...

class MIN2:
    r"""逐元素最小值指标 (MIN2)."""

    value: typing.Optional[float]
    def __new__(cls) -> "MIN2": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 MIN2 值
        """
        ...

class CLIP:
    r"""逐元素截断指标 (CLIP)."""

    value: typing.Optional[float]
    def __new__(cls) -> "CLIP": ...
    def update(
        self, value: float, min_value: float, max_value: float
    ) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 输入值
        :param min_value: 下界
        :param max_value: 上界
        :return: 当前 CLIP 值
        """
        ...

class ROUND:
    r"""逐元素四舍五入指标 (ROUND)."""

    value: typing.Optional[float]
    def __new__(cls) -> "ROUND": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 输入值
        :return: 当前 ROUND 值
        """
        ...

class POW:
    r"""逐元素幂运算指标 (POW)."""

    value: typing.Optional[float]
    def __new__(cls) -> "POW": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 POW 值
        """
        ...

class MOD:
    r"""逐元素取模指标 (MOD)."""

    value: typing.Optional[float]
    def __new__(cls) -> "MOD": ...
    def update(self, left: float, right: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param left: 第一个序列值
        :param right: 第二个序列值
        :return: 当前 MOD 值
        """
        ...

class CLAMP01:
    r"""固定区间截断指标 (CLAMP01)."""

    value: typing.Optional[float]
    def __new__(cls) -> "CLAMP01": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 输入值
        :return: 当前 CLAMP01 值
        """
        ...

class SQ:
    r"""逐元素平方指标 (SQ)."""

    value: typing.Optional[float]
    def __new__(cls) -> "SQ": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 输入值
        :return: 当前 SQ 值
        """
        ...

class CUBE:
    r"""逐元素立方指标 (CUBE)."""

    value: typing.Optional[float]
    def __new__(cls) -> "CUBE": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 输入值
        :return: 当前 CUBE 值
        """
        ...

class T3:
    r"""三重指数平滑指标 (T3)."""

    value: typing.Optional[float]
    def __new__(cls, period: int, vfactor: float) -> "T3": ...
    def update(self, value: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 T3 值
        """
        ...

class MAMA:
    r"""自适应均线指标 (MAMA)."""

    value: typing.Optional[tuple[float, float]]
    def __new__(cls, fast_limit: float, slow_limit: float) -> "MAMA": ...
    def update(self, value: float) -> typing.Optional[tuple[float, float]]:
        r"""
        更新指标值.

        :param value: 新数据点
        :return: 当前 (MAMA, FAMA)
        """
        ...

class WILLR:
    r"""威廉指标 (WILLR)."""

    value: typing.Optional[float]
    def __new__(cls, period: int) -> "WILLR": ...
    def update(self, high: float, low: float, close: float) -> typing.Optional[float]:
        r"""
        更新指标值.

        :param high: 最高价
        :param low: 最低价
        :param close: 收盘价
        :return: 当前 WILLR 值
        """
        ...

class StrategyContext:
    r"""
    策略上下文.

    :ivar orders: 订单列表 (内部使用)
    :ivar cash: 当前现金
    :ivar positions: 当前持仓
    :ivar available_positions: 可用持仓
    :ivar session: 当前交易时段
    """

    orders: list[Order]
    canceled_order_ids: list[str]
    active_orders: list[Order]
    timers: list[Timer]
    session: akquant.TradingSession
    current_time: int
    recent_trades: list[Trade]
    risk_config: RiskConfig
    strategy_id: typing.Optional[str]
    last_closed_trade: typing.Optional[ClosedTrade]
    closed_trades: list[ClosedTrade]
    cash: float
    positions: dict[str, float]
    available_positions: dict[str, float]
    def __new__(
        cls,
        cash: typing.Any,
        positions: typing.Mapping[str, float],
        available_positions: typing.Mapping[str, float],
        session: typing.Optional[akquant.TradingSession],
        current_time: typing.Optional[int],
        active_orders: typing.Optional[typing.Sequence[Order]],
        closed_trades: typing.Optional[typing.Sequence[ClosedTrade]],
        recent_trades: typing.Optional[typing.Sequence[Trade]],
        risk_config: typing.Optional[RiskConfig],
    ) -> "StrategyContext": ...
    def history(
        self, symbol: str, field: str, count: int
    ) -> typing.Optional[numpy.typing.NDArray[numpy.float64]]:
        r"""
        获取历史数据.

        :param symbol: 标的代码
        :param field: 字段名 (open, high, low, close, volume)
        :param count: 获取的数据长度
        :return: numpy array or None
        """
        ...

    def schedule(self, timestamp: int, payload: str) -> None:
        r"""
        注册定时器.

        :param timestamp: 触发时间戳 (纳秒)
        :param payload: 携带的数据 (如回调函数名)
        """
        ...

    def cancel_order(self, order_id: str) -> None:
        r"""
        取消订单.

        :param order_id: 订单 ID
        """
        ...

    def buy(
        self,
        symbol: str,
        quantity: float,
        price: typing.Optional[float] = ...,
        time_in_force: typing.Optional[akquant.TimeInForce] = ...,
        trigger_price: typing.Optional[float] = ...,
        tag: str = ...,
        order_type: typing.Optional[akquant.OrderType] = ...,
        trail_offset: typing.Optional[float] = ...,
        trail_reference_price: typing.Optional[float] = ...,
    ) -> str:
        r"""
        买入下单.

        :param symbol: 标的代码
        :param quantity: 买入数量 (正数)
        :param price: 限价 (可选, 默认为 Market 单)
        :param time_in_force: 订单有效期 (可选, 默认 GTC)
        :param trigger_price: 触发价格 (可选, 用于止损/止盈单)
        :param tag: 订单标签 (可选)
        :return: 订单 ID
        """
        ...

    def sell(
        self,
        symbol: str,
        quantity: float,
        price: typing.Optional[float] = ...,
        time_in_force: typing.Optional[akquant.TimeInForce] = ...,
        trigger_price: typing.Optional[float] = ...,
        tag: str = ...,
        order_type: typing.Optional[akquant.OrderType] = ...,
        trail_offset: typing.Optional[float] = ...,
        trail_reference_price: typing.Optional[float] = ...,
    ) -> str:
        r"""
        卖出下单.

        :param symbol: 标的代码
        :param quantity: 卖出数量 (正数)
        :param price: 限价 (可选, 默认为 Market 单)
        :param time_in_force: 订单有效期 (可选, 默认 GTC)
        :param trigger_price: 触发价格 (可选, 用于止损/止盈单)
        :param tag: 订单标签 (可选)
        :return: 订单 ID
        """
        ...

    def get_position(self, symbol: str) -> float:
        r"""
        获取当前持仓数量.

        :param symbol: 标的代码
        :return: 持仓数量 (Long为正, Short为负)
        """
        ...

    def get_available_position(self, symbol: str) -> float:
        r"""
        获取当前可用持仓数量.

        :param symbol: 标的代码
        :return: 可用持仓数量
        """
        ...

class Tick:
    r"""
    Tick 数据结构.

    :ivar timestamp: Unix 时间戳 (纳秒)
    :ivar price: 最新价
    :ivar volume: 成交量
    :ivar symbol: 标的代码
    """

    timestamp: int
    symbol: str
    price: float
    volume: float
    timestamp_str: str
    def __new__(
        cls, timestamp: typing.Any, price: typing.Any, volume: typing.Any, symbol: str
    ) -> "Tick": ...
    def set_price(self, value: typing.Any) -> None: ...
    def set_volume(self, value: typing.Any) -> None: ...
    def __repr__(self) -> str: ...

class Timer:
    timestamp: int
    payload: str

class Trade:
    r"""
    成交记录.

    :ivar id: 成交ID
    :ivar order_id: 订单ID
    :ivar symbol: 标的代码
    :ivar side: 交易方向
    :ivar quantity: 成交数量
    :ivar price: 成交价格
    :ivar commission: 手续费
    :ivar timestamp: Unix 时间戳 (纳秒)
    """

    id: str
    order_id: str
    symbol: str
    side: akquant.OrderSide
    timestamp: int
    bar_index: int
    quantity: float
    price: float
    commission: float
    owner_strategy_id: typing.Optional[str]
    def __new__(
        cls,
        id: str,
        order_id: str,
        symbol: str,
        side: akquant.OrderSide,
        quantity: typing.Any,
        price: typing.Any,
        commission: typing.Any,
        timestamp: int,
        bar_index: int,
        owner_strategy_id: typing.Optional[str] = ...,
    ) -> "Trade": ...
    def __repr__(self) -> str: ...

class TradePnL:
    r"""
    交易盈亏统计 (FIFO).

    :ivar gross_pnl: 总毛利 (扣除佣金前)
    :ivar net_pnl: 总净利 (扣除佣金后)
    :ivar total_commission: 总佣金
    :ivar total_closed_trades: 总平仓交易次数
    :ivar won_count: 盈利次数
    :ivar lost_count: 亏损次数
    :ivar won_pnl: 盈利总额
    :ivar lost_pnl: 亏损总额
    :ivar win_rate: 胜率 (0.0-1.0)
    :ivar loss_rate: 亏损率 (0.0-1.0)
    :ivar unrealized_pnl: 未实现盈亏
    :ivar avg_pnl: 平均单笔盈亏
    :ivar avg_return_pct: 平均单笔收益率 (%)
    :ivar avg_trade_bars: 平均持仓 K 线数
    :ivar avg_profit: 平均单笔盈利
    :ivar avg_profit_pct: 平均单笔盈利百分比
    :ivar avg_winning_trade_bars: 平均盈利交易持仓 K 线数
    :ivar avg_loss: 平均单笔亏损
    :ivar avg_loss_pct: 平均单笔亏损百分比
    :ivar avg_losing_trade_bars: 平均亏损交易持仓 K 线数
    :ivar largest_win: 最大单笔盈利
    :ivar largest_win_pct: 最大单笔盈利百分比
    :ivar largest_win_bars: 最大单笔盈利持仓 K 线数
    :ivar largest_loss: 最大单笔亏损
    :ivar largest_loss_pct: 最大单笔亏损百分比
    :ivar largest_loss_bars: 最大单笔亏损持仓 K 线数
    :ivar max_wins: 最大连胜次数
    :ivar max_losses: 最大连败次数
    :ivar profit_factor: 盈亏比
    :ivar total_profit: 总盈利 (同 won_pnl)
    :ivar total_loss: 总亏损 (同 lost_pnl)
    :ivar sqn: 系统质量数 (System Quality Number)
    :ivar kelly_criterion: 凯利公式比例
    """

    gross_pnl: float
    net_pnl: float
    total_commission: float
    total_closed_trades: int
    won_count: int
    lost_count: int
    won_pnl: float
    lost_pnl: float
    win_rate: float
    loss_rate: float
    unrealized_pnl: float
    avg_pnl: float
    avg_return_pct: float
    avg_trade_bars: float
    avg_profit: float
    avg_profit_pct: float
    avg_winning_trade_bars: float
    avg_loss: float
    avg_loss_pct: float
    avg_losing_trade_bars: float
    largest_win: float
    largest_win_pct: float
    largest_win_bars: float
    largest_loss: float
    largest_loss_pct: float
    largest_loss_bars: float
    max_wins: int
    max_losses: int
    profit_factor: float
    total_profit: float
    total_loss: float
    sqn: float
    kelly_criterion: float
    def __repr__(self) -> str: ...

def from_arrays(
    timestamps: typing.Any,
    opens: typing.Any,
    highs: typing.Any,
    lows: typing.Any,
    closes: typing.Any,
    volumes: typing.Any,
    symbol: typing.Optional[str],
    symbols: typing.Optional[typing.Sequence[str]],
    extra: typing.Optional[typing.Mapping[str, typing.Any]],
) -> list[Bar]:
    r"""
    从数组批量创建 Bar 列表 (Python 优化用 - Zero Copy).

    :param timestamps: 时间戳数组
    :param opens: 开盘价数组
    :param highs: 最高价数组
    :param lows: 最低价数组
    :param closes: 收盘价数组
    :param volumes: 成交量数组
    :param symbol: 标的代码 (可选)
    :param symbols: 标的代码数组 (可选)
    :param extra: 额外数据 (可选)
    """
    ...
